from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class AdjustmentEvent(__BaseDictObject):
    """
    An adjustment to the seller's account.
    """

    def __init__(self, data):
        super().__init__(data)
        if "AdjustmentType" in data:
            self.AdjustmentType: str = self._get_value(str, "AdjustmentType")
        else:
            self.AdjustmentType: str = None
        if "PostedDate" in data:
            self.PostedDate: Date = self._get_value(Date, "PostedDate")
        else:
            self.PostedDate: Date = None
        if "AdjustmentAmount" in data:
            self.AdjustmentAmount: Currency = self._get_value(Currency, "AdjustmentAmount")
        else:
            self.AdjustmentAmount: Currency = None
        if "AdjustmentItemList" in data:
            self.AdjustmentItemList: AdjustmentItemList = self._get_value(AdjustmentItemList, "AdjustmentItemList")
        else:
            self.AdjustmentItemList: AdjustmentItemList = None


class AdjustmentItem(__BaseDictObject):
    """
    An item in an adjustment to the seller's account.
    """

    def __init__(self, data):
        super().__init__(data)
        if "Quantity" in data:
            self.Quantity: str = self._get_value(str, "Quantity")
        else:
            self.Quantity: str = None
        if "PerUnitAmount" in data:
            self.PerUnitAmount: Currency = self._get_value(Currency, "PerUnitAmount")
        else:
            self.PerUnitAmount: Currency = None
        if "TotalAmount" in data:
            self.TotalAmount: Currency = self._get_value(Currency, "TotalAmount")
        else:
            self.TotalAmount: Currency = None
        if "SellerSKU" in data:
            self.SellerSKU: str = self._get_value(str, "SellerSKU")
        else:
            self.SellerSKU: str = None
        if "FnSKU" in data:
            self.FnSKU: str = self._get_value(str, "FnSKU")
        else:
            self.FnSKU: str = None
        if "ProductDescription" in data:
            self.ProductDescription: str = self._get_value(str, "ProductDescription")
        else:
            self.ProductDescription: str = None
        if "ASIN" in data:
            self.ASIN: str = self._get_value(str, "ASIN")
        else:
            self.ASIN: str = None


class AffordabilityExpenseEvent(__BaseDictObject):
    """
    An expense related to an affordability promotion.
    """

    def __init__(self, data):
        super().__init__(data)
        if "AmazonOrderId" in data:
            self.AmazonOrderId: str = self._get_value(str, "AmazonOrderId")
        else:
            self.AmazonOrderId: str = None
        if "PostedDate" in data:
            self.PostedDate: Date = self._get_value(Date, "PostedDate")
        else:
            self.PostedDate: Date = None
        if "MarketplaceId" in data:
            self.MarketplaceId: str = self._get_value(str, "MarketplaceId")
        else:
            self.MarketplaceId: str = None
        if "TransactionType" in data:
            self.TransactionType: str = self._get_value(str, "TransactionType")
        else:
            self.TransactionType: str = None
        if "BaseExpense" in data:
            self.BaseExpense: Currency = self._get_value(Currency, "BaseExpense")
        else:
            self.BaseExpense: Currency = None
        if "TaxTypeCGST" in data:
            self.TaxTypeCGST: Currency = self._get_value(Currency, "TaxTypeCGST")
        else:
            self.TaxTypeCGST: Currency = None
        if "TaxTypeSGST" in data:
            self.TaxTypeSGST: Currency = self._get_value(Currency, "TaxTypeSGST")
        else:
            self.TaxTypeSGST: Currency = None
        if "TaxTypeIGST" in data:
            self.TaxTypeIGST: Currency = self._get_value(Currency, "TaxTypeIGST")
        else:
            self.TaxTypeIGST: Currency = None
        if "TotalExpense" in data:
            self.TotalExpense: Currency = self._get_value(Currency, "TotalExpense")
        else:
            self.TotalExpense: Currency = None


class ChargeComponent(__BaseDictObject):
    """
        A charge on the seller's account.
    Possible values:
    * Principal - The selling price of the order item, equal to the selling price of the item multiplied by the quantity ordered.
    * Tax - The tax collected by the seller on the Principal.
    * MarketplaceFacilitatorTax-Principal - The tax withheld on the Principal.
    * MarketplaceFacilitatorTax-Shipping - The tax withheld on the ShippingCharge.
    * MarketplaceFacilitatorTax-Giftwrap - The tax withheld on the Giftwrap charge.
    * MarketplaceFacilitatorTax-Other - The tax withheld on other miscellaneous charges.
    * Discount - The promotional discount for an order item.
    * TaxDiscount - The tax amount deducted for promotional rebates.
    * CODItemCharge - The COD charge for an order item.
    * CODItemTaxCharge - The tax collected by the seller on a CODItemCharge.
    * CODOrderCharge - The COD charge for an order.
    * CODOrderTaxCharge - The tax collected by the seller on a CODOrderCharge.
    * CODShippingCharge - Shipping charges for a COD order.
    * CODShippingTaxCharge - The tax collected by the seller on a CODShippingCharge.
    * ShippingCharge - The shipping charge.
    * ShippingTax - The tax collected by the seller on a ShippingCharge.
    * Goodwill - The amount given to a buyer as a gesture of goodwill or to compensate for pain and suffering in the buying experience.
    * Giftwrap - The gift wrap charge.
    * GiftwrapTax - The tax collected by the seller on a Giftwrap charge.
    * RestockingFee - The charge applied to the buyer when returning a product in certain categories.
    * ReturnShipping - The amount given to the buyer to compensate for shipping the item back in the event we are at fault.
    * PointsFee - The value of Amazon Points deducted from the refund if the buyer does not have enough Amazon Points to cover the deduction.
    * GenericDeduction - A generic bad debt deduction.
    * FreeReplacementReturnShipping - The compensation for return shipping when a buyer receives the wrong item, requests a free replacement, and returns the incorrect item.
    * PaymentMethodFee - The fee collected for certain payment methods in certain marketplaces.
    * ExportCharge - The export duty that is charged when an item is shipped to an international destination as part of the Amazon Global program.
    * SAFE-TReimbursement - The SAFE-T claim amount for the item.
    * TCS-CGST - Tax Collected at Source (TCS) for Central Goods and Services Tax (CGST).
    * TCS-SGST - Tax Collected at Source for State Goods and Services Tax (SGST).
    * TCS-IGST - Tax Collected at Source for Integrated Goods and Services Tax (IGST).
    * TCS-UTGST - Tax Collected at Source for Union Territories Goods and Services Tax (UTGST).
    """

    def __init__(self, data):
        super().__init__(data)
        if "ChargeType" in data:
            self.ChargeType: str = self._get_value(str, "ChargeType")
        else:
            self.ChargeType: str = None
        if "ChargeAmount" in data:
            self.ChargeAmount: Currency = self._get_value(Currency, "ChargeAmount")
        else:
            self.ChargeAmount: Currency = None


class ChargeInstrument(__BaseDictObject):
    """
    A payment instrument.
    """

    def __init__(self, data):
        super().__init__(data)
        if "Description" in data:
            self.Description: str = self._get_value(str, "Description")
        else:
            self.Description: str = None
        if "Tail" in data:
            self.Tail: str = self._get_value(str, "Tail")
        else:
            self.Tail: str = None
        if "Amount" in data:
            self.Amount: Currency = self._get_value(Currency, "Amount")
        else:
            self.Amount: Currency = None


class CouponPaymentEvent(__BaseDictObject):
    """
    An event related to coupon payments.
    """

    def __init__(self, data):
        super().__init__(data)
        if "PostedDate" in data:
            self.PostedDate: Date = self._get_value(Date, "PostedDate")
        else:
            self.PostedDate: Date = None
        if "CouponId" in data:
            self.CouponId: str = self._get_value(str, "CouponId")
        else:
            self.CouponId: str = None
        if "SellerCouponDescription" in data:
            self.SellerCouponDescription: str = self._get_value(str, "SellerCouponDescription")
        else:
            self.SellerCouponDescription: str = None
        if "ClipOrRedemptionCount" in data:
            self.ClipOrRedemptionCount: int = self._get_value(int, "ClipOrRedemptionCount")
        else:
            self.ClipOrRedemptionCount: int = None
        if "PaymentEventId" in data:
            self.PaymentEventId: str = self._get_value(str, "PaymentEventId")
        else:
            self.PaymentEventId: str = None
        if "FeeComponent" in data:
            self.FeeComponent: FeeComponent = self._get_value(FeeComponent, "FeeComponent")
        else:
            self.FeeComponent: FeeComponent = None
        if "ChargeComponent" in data:
            self.ChargeComponent: ChargeComponent = self._get_value(ChargeComponent, "ChargeComponent")
        else:
            self.ChargeComponent: ChargeComponent = None
        if "TotalAmount" in data:
            self.TotalAmount: Currency = self._get_value(Currency, "TotalAmount")
        else:
            self.TotalAmount: Currency = None


class Currency(__BaseDictObject):
    """
    A currency type and amount.
    """

    def __init__(self, data):
        super().__init__(data)
        if "CurrencyCode" in data:
            self.CurrencyCode: str = self._get_value(str, "CurrencyCode")
        else:
            self.CurrencyCode: str = None
        if "CurrencyAmount" in data:
            self.CurrencyAmount: BigDecimal = self._get_value(BigDecimal, "CurrencyAmount")
        else:
            self.CurrencyAmount: BigDecimal = None


class DebtRecoveryEvent(__BaseDictObject):
    """
    A debt payment or debt adjustment.
    """

    def __init__(self, data):
        super().__init__(data)
        if "DebtRecoveryType" in data:
            self.DebtRecoveryType: str = self._get_value(str, "DebtRecoveryType")
        else:
            self.DebtRecoveryType: str = None
        if "RecoveryAmount" in data:
            self.RecoveryAmount: Currency = self._get_value(Currency, "RecoveryAmount")
        else:
            self.RecoveryAmount: Currency = None
        if "OverPaymentCredit" in data:
            self.OverPaymentCredit: Currency = self._get_value(Currency, "OverPaymentCredit")
        else:
            self.OverPaymentCredit: Currency = None
        if "DebtRecoveryItemList" in data:
            self.DebtRecoveryItemList: DebtRecoveryItemList = self._get_value(
                DebtRecoveryItemList, "DebtRecoveryItemList"
            )
        else:
            self.DebtRecoveryItemList: DebtRecoveryItemList = None
        if "ChargeInstrumentList" in data:
            self.ChargeInstrumentList: ChargeInstrumentList = self._get_value(
                ChargeInstrumentList, "ChargeInstrumentList"
            )
        else:
            self.ChargeInstrumentList: ChargeInstrumentList = None


class DebtRecoveryItem(__BaseDictObject):
    """
    An item of a debt payment or debt adjustment.
    """

    def __init__(self, data):
        super().__init__(data)
        if "RecoveryAmount" in data:
            self.RecoveryAmount: Currency = self._get_value(Currency, "RecoveryAmount")
        else:
            self.RecoveryAmount: Currency = None
        if "OriginalAmount" in data:
            self.OriginalAmount: Currency = self._get_value(Currency, "OriginalAmount")
        else:
            self.OriginalAmount: Currency = None
        if "GroupBeginDate" in data:
            self.GroupBeginDate: Date = self._get_value(Date, "GroupBeginDate")
        else:
            self.GroupBeginDate: Date = None
        if "GroupEndDate" in data:
            self.GroupEndDate: Date = self._get_value(Date, "GroupEndDate")
        else:
            self.GroupEndDate: Date = None


class DirectPayment(__BaseDictObject):
    """
    A payment made directly to a seller.
    """

    def __init__(self, data):
        super().__init__(data)
        if "DirectPaymentType" in data:
            self.DirectPaymentType: str = self._get_value(str, "DirectPaymentType")
        else:
            self.DirectPaymentType: str = None
        if "DirectPaymentAmount" in data:
            self.DirectPaymentAmount: Currency = self._get_value(Currency, "DirectPaymentAmount")
        else:
            self.DirectPaymentAmount: Currency = None


class FBALiquidationEvent(__BaseDictObject):
    """
    A payment event for Fulfillment by Amazon (FBA) inventory liquidation. This event is used only in the US marketplace.
    """

    def __init__(self, data):
        super().__init__(data)
        if "PostedDate" in data:
            self.PostedDate: Date = self._get_value(Date, "PostedDate")
        else:
            self.PostedDate: Date = None
        if "OriginalRemovalOrderId" in data:
            self.OriginalRemovalOrderId: str = self._get_value(str, "OriginalRemovalOrderId")
        else:
            self.OriginalRemovalOrderId: str = None
        if "LiquidationProceedsAmount" in data:
            self.LiquidationProceedsAmount: Currency = self._get_value(Currency, "LiquidationProceedsAmount")
        else:
            self.LiquidationProceedsAmount: Currency = None
        if "LiquidationFeeAmount" in data:
            self.LiquidationFeeAmount: Currency = self._get_value(Currency, "LiquidationFeeAmount")
        else:
            self.LiquidationFeeAmount: Currency = None


class FeeComponent(__BaseDictObject):
    """
    A fee associated with the event.
    """

    def __init__(self, data):
        super().__init__(data)
        if "FeeType" in data:
            self.FeeType: str = self._get_value(str, "FeeType")
        else:
            self.FeeType: str = None
        if "FeeAmount" in data:
            self.FeeAmount: Currency = self._get_value(Currency, "FeeAmount")
        else:
            self.FeeAmount: Currency = None


class FinancialEventGroup(__BaseDictObject):
    """
    Information related to a financial event group.
    """

    def __init__(self, data):
        super().__init__(data)
        if "FinancialEventGroupId" in data:
            self.FinancialEventGroupId: str = self._get_value(str, "FinancialEventGroupId")
        else:
            self.FinancialEventGroupId: str = None
        if "ProcessingStatus" in data:
            self.ProcessingStatus: str = self._get_value(str, "ProcessingStatus")
        else:
            self.ProcessingStatus: str = None
        if "FundTransferStatus" in data:
            self.FundTransferStatus: str = self._get_value(str, "FundTransferStatus")
        else:
            self.FundTransferStatus: str = None
        if "OriginalTotal" in data:
            self.OriginalTotal: Currency = self._get_value(Currency, "OriginalTotal")
        else:
            self.OriginalTotal: Currency = None
        if "ConvertedTotal" in data:
            self.ConvertedTotal: Currency = self._get_value(Currency, "ConvertedTotal")
        else:
            self.ConvertedTotal: Currency = None
        if "FundTransferDate" in data:
            self.FundTransferDate: Date = self._get_value(Date, "FundTransferDate")
        else:
            self.FundTransferDate: Date = None
        if "TraceId" in data:
            self.TraceId: str = self._get_value(str, "TraceId")
        else:
            self.TraceId: str = None
        if "AccountTail" in data:
            self.AccountTail: str = self._get_value(str, "AccountTail")
        else:
            self.AccountTail: str = None
        if "BeginningBalance" in data:
            self.BeginningBalance: Currency = self._get_value(Currency, "BeginningBalance")
        else:
            self.BeginningBalance: Currency = None
        if "FinancialEventGroupStart" in data:
            self.FinancialEventGroupStart: Date = self._get_value(Date, "FinancialEventGroupStart")
        else:
            self.FinancialEventGroupStart: Date = None
        if "FinancialEventGroupEnd" in data:
            self.FinancialEventGroupEnd: Date = self._get_value(Date, "FinancialEventGroupEnd")
        else:
            self.FinancialEventGroupEnd: Date = None


class FinancialEvents(__BaseDictObject):
    """
    Contains all information related to a financial event.
    """

    def __init__(self, data):
        super().__init__(data)
        if "ShipmentEventList" in data:
            self.ShipmentEventList: ShipmentEventList = self._get_value(ShipmentEventList, "ShipmentEventList")
        else:
            self.ShipmentEventList: ShipmentEventList = None
        if "RefundEventList" in data:
            self.RefundEventList: ShipmentEventList = self._get_value(ShipmentEventList, "RefundEventList")
        else:
            self.RefundEventList: ShipmentEventList = None
        if "GuaranteeClaimEventList" in data:
            self.GuaranteeClaimEventList: ShipmentEventList = self._get_value(
                ShipmentEventList, "GuaranteeClaimEventList"
            )
        else:
            self.GuaranteeClaimEventList: ShipmentEventList = None
        if "ChargebackEventList" in data:
            self.ChargebackEventList: ShipmentEventList = self._get_value(ShipmentEventList, "ChargebackEventList")
        else:
            self.ChargebackEventList: ShipmentEventList = None
        if "PayWithAmazonEventList" in data:
            self.PayWithAmazonEventList: PayWithAmazonEventList = self._get_value(
                PayWithAmazonEventList, "PayWithAmazonEventList"
            )
        else:
            self.PayWithAmazonEventList: PayWithAmazonEventList = None
        if "ServiceProviderCreditEventList" in data:
            self.ServiceProviderCreditEventList: SolutionProviderCreditEventList = self._get_value(
                SolutionProviderCreditEventList, "ServiceProviderCreditEventList"
            )
        else:
            self.ServiceProviderCreditEventList: SolutionProviderCreditEventList = None
        if "RetrochargeEventList" in data:
            self.RetrochargeEventList: RetrochargeEventList = self._get_value(
                RetrochargeEventList, "RetrochargeEventList"
            )
        else:
            self.RetrochargeEventList: RetrochargeEventList = None
        if "RentalTransactionEventList" in data:
            self.RentalTransactionEventList: RentalTransactionEventList = self._get_value(
                RentalTransactionEventList, "RentalTransactionEventList"
            )
        else:
            self.RentalTransactionEventList: RentalTransactionEventList = None
        if "ProductAdsPaymentEventList" in data:
            self.ProductAdsPaymentEventList: ProductAdsPaymentEventList = self._get_value(
                ProductAdsPaymentEventList, "ProductAdsPaymentEventList"
            )
        else:
            self.ProductAdsPaymentEventList: ProductAdsPaymentEventList = None
        if "ServiceFeeEventList" in data:
            self.ServiceFeeEventList: ServiceFeeEventList = self._get_value(ServiceFeeEventList, "ServiceFeeEventList")
        else:
            self.ServiceFeeEventList: ServiceFeeEventList = None
        if "SellerDealPaymentEventList" in data:
            self.SellerDealPaymentEventList: SellerDealPaymentEventList = self._get_value(
                SellerDealPaymentEventList, "SellerDealPaymentEventList"
            )
        else:
            self.SellerDealPaymentEventList: SellerDealPaymentEventList = None
        if "DebtRecoveryEventList" in data:
            self.DebtRecoveryEventList: DebtRecoveryEventList = self._get_value(
                DebtRecoveryEventList, "DebtRecoveryEventList"
            )
        else:
            self.DebtRecoveryEventList: DebtRecoveryEventList = None
        if "LoanServicingEventList" in data:
            self.LoanServicingEventList: LoanServicingEventList = self._get_value(
                LoanServicingEventList, "LoanServicingEventList"
            )
        else:
            self.LoanServicingEventList: LoanServicingEventList = None
        if "AdjustmentEventList" in data:
            self.AdjustmentEventList: AdjustmentEventList = self._get_value(AdjustmentEventList, "AdjustmentEventList")
        else:
            self.AdjustmentEventList: AdjustmentEventList = None
        if "SAFETReimbursementEventList" in data:
            self.SAFETReimbursementEventList: SAFETReimbursementEventList = self._get_value(
                SAFETReimbursementEventList, "SAFETReimbursementEventList"
            )
        else:
            self.SAFETReimbursementEventList: SAFETReimbursementEventList = None
        if "SellerReviewEnrollmentPaymentEventList" in data:
            self.SellerReviewEnrollmentPaymentEventList: SellerReviewEnrollmentPaymentEventList = self._get_value(
                SellerReviewEnrollmentPaymentEventList, "SellerReviewEnrollmentPaymentEventList"
            )
        else:
            self.SellerReviewEnrollmentPaymentEventList: SellerReviewEnrollmentPaymentEventList = None
        if "FBALiquidationEventList" in data:
            self.FBALiquidationEventList: FBALiquidationEventList = self._get_value(
                FBALiquidationEventList, "FBALiquidationEventList"
            )
        else:
            self.FBALiquidationEventList: FBALiquidationEventList = None
        if "CouponPaymentEventList" in data:
            self.CouponPaymentEventList: CouponPaymentEventList = self._get_value(
                CouponPaymentEventList, "CouponPaymentEventList"
            )
        else:
            self.CouponPaymentEventList: CouponPaymentEventList = None
        if "ImagingServicesFeeEventList" in data:
            self.ImagingServicesFeeEventList: ImagingServicesFeeEventList = self._get_value(
                ImagingServicesFeeEventList, "ImagingServicesFeeEventList"
            )
        else:
            self.ImagingServicesFeeEventList: ImagingServicesFeeEventList = None
        if "NetworkComminglingTransactionEventList" in data:
            self.NetworkComminglingTransactionEventList: NetworkComminglingTransactionEventList = self._get_value(
                NetworkComminglingTransactionEventList, "NetworkComminglingTransactionEventList"
            )
        else:
            self.NetworkComminglingTransactionEventList: NetworkComminglingTransactionEventList = None
        if "AffordabilityExpenseEventList" in data:
            self.AffordabilityExpenseEventList: AffordabilityExpenseEventList = self._get_value(
                AffordabilityExpenseEventList, "AffordabilityExpenseEventList"
            )
        else:
            self.AffordabilityExpenseEventList: AffordabilityExpenseEventList = None
        if "AffordabilityExpenseReversalEventList" in data:
            self.AffordabilityExpenseReversalEventList: AffordabilityExpenseEventList = self._get_value(
                AffordabilityExpenseEventList, "AffordabilityExpenseReversalEventList"
            )
        else:
            self.AffordabilityExpenseReversalEventList: AffordabilityExpenseEventList = None
        if "TrialShipmentEventList" in data:
            self.TrialShipmentEventList: TrialShipmentEventList = self._get_value(
                TrialShipmentEventList, "TrialShipmentEventList"
            )
        else:
            self.TrialShipmentEventList: TrialShipmentEventList = None
        if "ShipmentSettleEventList" in data:
            self.ShipmentSettleEventList: ShipmentSettleEventList = self._get_value(
                ShipmentSettleEventList, "ShipmentSettleEventList"
            )
        else:
            self.ShipmentSettleEventList: ShipmentSettleEventList = None
        if "TaxWithholdingEventList" in data:
            self.TaxWithholdingEventList: TaxWithholdingEventList = self._get_value(
                TaxWithholdingEventList, "TaxWithholdingEventList"
            )
        else:
            self.TaxWithholdingEventList: TaxWithholdingEventList = None
        if "RemovalShipmentEventList" in data:
            self.RemovalShipmentEventList: RemovalShipmentEventList = self._get_value(
                RemovalShipmentEventList, "RemovalShipmentEventList"
            )
        else:
            self.RemovalShipmentEventList: RemovalShipmentEventList = None
        if "RemovalShipmentAdjustmentEventList" in data:
            self.RemovalShipmentAdjustmentEventList: RemovalShipmentAdjustmentEventList = self._get_value(
                RemovalShipmentAdjustmentEventList, "RemovalShipmentAdjustmentEventList"
            )
        else:
            self.RemovalShipmentAdjustmentEventList: RemovalShipmentAdjustmentEventList = None


class ImagingServicesFeeEvent(__BaseDictObject):
    """
    A fee event related to Amazon Imaging services.
    """

    def __init__(self, data):
        super().__init__(data)
        if "ImagingRequestBillingItemID" in data:
            self.ImagingRequestBillingItemID: str = self._get_value(str, "ImagingRequestBillingItemID")
        else:
            self.ImagingRequestBillingItemID: str = None
        if "ASIN" in data:
            self.ASIN: str = self._get_value(str, "ASIN")
        else:
            self.ASIN: str = None
        if "PostedDate" in data:
            self.PostedDate: Date = self._get_value(Date, "PostedDate")
        else:
            self.PostedDate: Date = None
        if "FeeList" in data:
            self.FeeList: FeeComponentList = self._get_value(FeeComponentList, "FeeList")
        else:
            self.FeeList: FeeComponentList = None


class ListFinancialEventGroupsPayload(__BaseDictObject):
    """
    The payload for the listFinancialEventGroups operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "NextToken" in data:
            self.NextToken: str = self._get_value(str, "NextToken")
        else:
            self.NextToken: str = None
        if "FinancialEventGroupList" in data:
            self.FinancialEventGroupList: FinancialEventGroupList = self._get_value(
                FinancialEventGroupList, "FinancialEventGroupList"
            )
        else:
            self.FinancialEventGroupList: FinancialEventGroupList = None


class ListFinancialEventGroupsResponse(__BaseDictObject):
    """
    The response schema for the listFinancialEventGroups operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: ListFinancialEventGroupsPayload = self._get_value(ListFinancialEventGroupsPayload, "payload")
        else:
            self.payload: ListFinancialEventGroupsPayload = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class ListFinancialEventsPayload(__BaseDictObject):
    """
    The payload for the listFinancialEvents operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "NextToken" in data:
            self.NextToken: str = self._get_value(str, "NextToken")
        else:
            self.NextToken: str = None
        if "FinancialEvents" in data:
            self.FinancialEvents: FinancialEvents = self._get_value(FinancialEvents, "FinancialEvents")
        else:
            self.FinancialEvents: FinancialEvents = None


class ListFinancialEventsResponse(__BaseDictObject):
    """
    The response schema for the listFinancialEvents operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: ListFinancialEventsPayload = self._get_value(ListFinancialEventsPayload, "payload")
        else:
            self.payload: ListFinancialEventsPayload = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class LoanServicingEvent(__BaseDictObject):
    """
    A loan advance, loan payment, or loan refund.
    """

    def __init__(self, data):
        super().__init__(data)
        if "LoanAmount" in data:
            self.LoanAmount: Currency = self._get_value(Currency, "LoanAmount")
        else:
            self.LoanAmount: Currency = None
        if "SourceBusinessEventType" in data:
            self.SourceBusinessEventType: str = self._get_value(str, "SourceBusinessEventType")
        else:
            self.SourceBusinessEventType: str = None


class NetworkComminglingTransactionEvent(__BaseDictObject):
    """
    A network commingling transaction event.
    """

    def __init__(self, data):
        super().__init__(data)
        if "TransactionType" in data:
            self.TransactionType: str = self._get_value(str, "TransactionType")
        else:
            self.TransactionType: str = None
        if "PostedDate" in data:
            self.PostedDate: Date = self._get_value(Date, "PostedDate")
        else:
            self.PostedDate: Date = None
        if "NetCoTransactionID" in data:
            self.NetCoTransactionID: str = self._get_value(str, "NetCoTransactionID")
        else:
            self.NetCoTransactionID: str = None
        if "SwapReason" in data:
            self.SwapReason: str = self._get_value(str, "SwapReason")
        else:
            self.SwapReason: str = None
        if "ASIN" in data:
            self.ASIN: str = self._get_value(str, "ASIN")
        else:
            self.ASIN: str = None
        if "MarketplaceId" in data:
            self.MarketplaceId: str = self._get_value(str, "MarketplaceId")
        else:
            self.MarketplaceId: str = None
        if "TaxExclusiveAmount" in data:
            self.TaxExclusiveAmount: Currency = self._get_value(Currency, "TaxExclusiveAmount")
        else:
            self.TaxExclusiveAmount: Currency = None
        if "TaxAmount" in data:
            self.TaxAmount: Currency = self._get_value(Currency, "TaxAmount")
        else:
            self.TaxAmount: Currency = None


class PayWithAmazonEvent(__BaseDictObject):
    """
    An event related to the seller's Pay with Amazon account.
    """

    def __init__(self, data):
        super().__init__(data)
        if "SellerOrderId" in data:
            self.SellerOrderId: str = self._get_value(str, "SellerOrderId")
        else:
            self.SellerOrderId: str = None
        if "TransactionPostedDate" in data:
            self.TransactionPostedDate: Date = self._get_value(Date, "TransactionPostedDate")
        else:
            self.TransactionPostedDate: Date = None
        if "BusinessObjectType" in data:
            self.BusinessObjectType: str = self._get_value(str, "BusinessObjectType")
        else:
            self.BusinessObjectType: str = None
        if "SalesChannel" in data:
            self.SalesChannel: str = self._get_value(str, "SalesChannel")
        else:
            self.SalesChannel: str = None
        if "Charge" in data:
            self.Charge: ChargeComponent = self._get_value(ChargeComponent, "Charge")
        else:
            self.Charge: ChargeComponent = None
        if "FeeList" in data:
            self.FeeList: FeeComponentList = self._get_value(FeeComponentList, "FeeList")
        else:
            self.FeeList: FeeComponentList = None
        if "PaymentAmountType" in data:
            self.PaymentAmountType: str = self._get_value(str, "PaymentAmountType")
        else:
            self.PaymentAmountType: str = None
        if "AmountDescription" in data:
            self.AmountDescription: str = self._get_value(str, "AmountDescription")
        else:
            self.AmountDescription: str = None
        if "FulfillmentChannel" in data:
            self.FulfillmentChannel: str = self._get_value(str, "FulfillmentChannel")
        else:
            self.FulfillmentChannel: str = None
        if "StoreName" in data:
            self.StoreName: str = self._get_value(str, "StoreName")
        else:
            self.StoreName: str = None


class ProductAdsPaymentEvent(__BaseDictObject):
    """
    A Sponsored Products payment event.
    """

    def __init__(self, data):
        super().__init__(data)
        if "postedDate" in data:
            self.postedDate: Date = self._get_value(Date, "postedDate")
        else:
            self.postedDate: Date = None
        if "transactionType" in data:
            self.transactionType: str = self._get_value(str, "transactionType")
        else:
            self.transactionType: str = None
        if "invoiceId" in data:
            self.invoiceId: str = self._get_value(str, "invoiceId")
        else:
            self.invoiceId: str = None
        if "baseValue" in data:
            self.baseValue: Currency = self._get_value(Currency, "baseValue")
        else:
            self.baseValue: Currency = None
        if "taxValue" in data:
            self.taxValue: Currency = self._get_value(Currency, "taxValue")
        else:
            self.taxValue: Currency = None
        if "transactionValue" in data:
            self.transactionValue: Currency = self._get_value(Currency, "transactionValue")
        else:
            self.transactionValue: Currency = None


class Promotion(__BaseDictObject):
    """
    A promotion applied to an item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "PromotionType" in data:
            self.PromotionType: str = self._get_value(str, "PromotionType")
        else:
            self.PromotionType: str = None
        if "PromotionId" in data:
            self.PromotionId: str = self._get_value(str, "PromotionId")
        else:
            self.PromotionId: str = None
        if "PromotionAmount" in data:
            self.PromotionAmount: Currency = self._get_value(Currency, "PromotionAmount")
        else:
            self.PromotionAmount: Currency = None


class RemovalShipmentEvent(__BaseDictObject):
    """
    A removal shipment event for a removal order.
    """

    def __init__(self, data):
        super().__init__(data)
        if "PostedDate" in data:
            self.PostedDate: Date = self._get_value(Date, "PostedDate")
        else:
            self.PostedDate: Date = None
        if "MerchantOrderId" in data:
            self.MerchantOrderId: str = self._get_value(str, "MerchantOrderId")
        else:
            self.MerchantOrderId: str = None
        if "OrderId" in data:
            self.OrderId: str = self._get_value(str, "OrderId")
        else:
            self.OrderId: str = None
        if "TransactionType" in data:
            self.TransactionType: str = self._get_value(str, "TransactionType")
        else:
            self.TransactionType: str = None
        if "RemovalShipmentItemList" in data:
            self.RemovalShipmentItemList: RemovalShipmentItemList = self._get_value(
                RemovalShipmentItemList, "RemovalShipmentItemList"
            )
        else:
            self.RemovalShipmentItemList: RemovalShipmentItemList = None


class RemovalShipmentItem(__BaseDictObject):
    """
    Item-level information for a removal shipment.
    """

    def __init__(self, data):
        super().__init__(data)
        if "RemovalShipmentItemId" in data:
            self.RemovalShipmentItemId: str = self._get_value(str, "RemovalShipmentItemId")
        else:
            self.RemovalShipmentItemId: str = None
        if "TaxCollectionModel" in data:
            self.TaxCollectionModel: str = self._get_value(str, "TaxCollectionModel")
        else:
            self.TaxCollectionModel: str = None
        if "FulfillmentNetworkSKU" in data:
            self.FulfillmentNetworkSKU: str = self._get_value(str, "FulfillmentNetworkSKU")
        else:
            self.FulfillmentNetworkSKU: str = None
        if "Quantity" in data:
            self.Quantity: int = self._get_value(int, "Quantity")
        else:
            self.Quantity: int = None
        if "Revenue" in data:
            self.Revenue: Currency = self._get_value(Currency, "Revenue")
        else:
            self.Revenue: Currency = None
        if "FeeAmount" in data:
            self.FeeAmount: Currency = self._get_value(Currency, "FeeAmount")
        else:
            self.FeeAmount: Currency = None
        if "TaxAmount" in data:
            self.TaxAmount: Currency = self._get_value(Currency, "TaxAmount")
        else:
            self.TaxAmount: Currency = None
        if "TaxWithheld" in data:
            self.TaxWithheld: Currency = self._get_value(Currency, "TaxWithheld")
        else:
            self.TaxWithheld: Currency = None


class RemovalShipmentAdjustmentEvent(__BaseDictObject):
    """
    A financial adjustment event for FBA liquidated inventory. A positive value indicates money owed to Amazon by the buyer (for example, when the charge was incorrectly calculated as less than it should be). A negative value indicates a full or partial refund owed to the buyer (for example, when the buyer receives damaged items or fewer items than ordered).
    """

    def __init__(self, data):
        super().__init__(data)
        if "PostedDate" in data:
            self.PostedDate: Date = self._get_value(Date, "PostedDate")
        else:
            self.PostedDate: Date = None
        if "AdjustmentEventId" in data:
            self.AdjustmentEventId: str = self._get_value(str, "AdjustmentEventId")
        else:
            self.AdjustmentEventId: str = None
        if "MerchantOrderId" in data:
            self.MerchantOrderId: str = self._get_value(str, "MerchantOrderId")
        else:
            self.MerchantOrderId: str = None
        if "OrderId" in data:
            self.OrderId: str = self._get_value(str, "OrderId")
        else:
            self.OrderId: str = None
        if "TransactionType" in data:
            self.TransactionType: str = self._get_value(str, "TransactionType")
        else:
            self.TransactionType: str = None
        if "RemovalShipmentItemAdjustmentList" in data:
            self.RemovalShipmentItemAdjustmentList: _List[RemovalShipmentItemAdjustment] = [
                RemovalShipmentItemAdjustment(datum) for datum in data["RemovalShipmentItemAdjustmentList"]
            ]
        else:
            self.RemovalShipmentItemAdjustmentList: _List[RemovalShipmentItemAdjustment] = []


class RemovalShipmentItemAdjustment(__BaseDictObject):
    """
    Item-level information for a removal shipment item adjustment.
    """

    def __init__(self, data):
        super().__init__(data)
        if "RemovalShipmentItemId" in data:
            self.RemovalShipmentItemId: str = self._get_value(str, "RemovalShipmentItemId")
        else:
            self.RemovalShipmentItemId: str = None
        if "TaxCollectionModel" in data:
            self.TaxCollectionModel: str = self._get_value(str, "TaxCollectionModel")
        else:
            self.TaxCollectionModel: str = None
        if "FulfillmentNetworkSKU" in data:
            self.FulfillmentNetworkSKU: str = self._get_value(str, "FulfillmentNetworkSKU")
        else:
            self.FulfillmentNetworkSKU: str = None
        if "AdjustedQuantity" in data:
            self.AdjustedQuantity: int = self._get_value(int, "AdjustedQuantity")
        else:
            self.AdjustedQuantity: int = None
        if "RevenueAdjustment" in data:
            self.RevenueAdjustment: Currency = self._get_value(Currency, "RevenueAdjustment")
        else:
            self.RevenueAdjustment: Currency = None
        if "TaxAmountAdjustment" in data:
            self.TaxAmountAdjustment: Currency = self._get_value(Currency, "TaxAmountAdjustment")
        else:
            self.TaxAmountAdjustment: Currency = None
        if "TaxWithheldAdjustment" in data:
            self.TaxWithheldAdjustment: Currency = self._get_value(Currency, "TaxWithheldAdjustment")
        else:
            self.TaxWithheldAdjustment: Currency = None


class RentalTransactionEvent(__BaseDictObject):
    """
    An event related to a rental transaction.
    """

    def __init__(self, data):
        super().__init__(data)
        if "AmazonOrderId" in data:
            self.AmazonOrderId: str = self._get_value(str, "AmazonOrderId")
        else:
            self.AmazonOrderId: str = None
        if "RentalEventType" in data:
            self.RentalEventType: str = self._get_value(str, "RentalEventType")
        else:
            self.RentalEventType: str = None
        if "ExtensionLength" in data:
            self.ExtensionLength: int = self._get_value(int, "ExtensionLength")
        else:
            self.ExtensionLength: int = None
        if "PostedDate" in data:
            self.PostedDate: Date = self._get_value(Date, "PostedDate")
        else:
            self.PostedDate: Date = None
        if "RentalChargeList" in data:
            self.RentalChargeList: ChargeComponentList = self._get_value(ChargeComponentList, "RentalChargeList")
        else:
            self.RentalChargeList: ChargeComponentList = None
        if "RentalFeeList" in data:
            self.RentalFeeList: FeeComponentList = self._get_value(FeeComponentList, "RentalFeeList")
        else:
            self.RentalFeeList: FeeComponentList = None
        if "MarketplaceName" in data:
            self.MarketplaceName: str = self._get_value(str, "MarketplaceName")
        else:
            self.MarketplaceName: str = None
        if "RentalInitialValue" in data:
            self.RentalInitialValue: Currency = self._get_value(Currency, "RentalInitialValue")
        else:
            self.RentalInitialValue: Currency = None
        if "RentalReimbursement" in data:
            self.RentalReimbursement: Currency = self._get_value(Currency, "RentalReimbursement")
        else:
            self.RentalReimbursement: Currency = None
        if "RentalTaxWithheldList" in data:
            self.RentalTaxWithheldList: TaxWithheldComponentList = self._get_value(
                TaxWithheldComponentList, "RentalTaxWithheldList"
            )
        else:
            self.RentalTaxWithheldList: TaxWithheldComponentList = None


class RetrochargeEvent(__BaseDictObject):
    """
    A retrocharge or retrocharge reversal.
    """

    def __init__(self, data):
        super().__init__(data)
        if "RetrochargeEventType" in data:
            self.RetrochargeEventType: str = self._get_value(str, "RetrochargeEventType")
        else:
            self.RetrochargeEventType: str = None
        if "AmazonOrderId" in data:
            self.AmazonOrderId: str = self._get_value(str, "AmazonOrderId")
        else:
            self.AmazonOrderId: str = None
        if "PostedDate" in data:
            self.PostedDate: Date = self._get_value(Date, "PostedDate")
        else:
            self.PostedDate: Date = None
        if "BaseTax" in data:
            self.BaseTax: Currency = self._get_value(Currency, "BaseTax")
        else:
            self.BaseTax: Currency = None
        if "ShippingTax" in data:
            self.ShippingTax: Currency = self._get_value(Currency, "ShippingTax")
        else:
            self.ShippingTax: Currency = None
        if "MarketplaceName" in data:
            self.MarketplaceName: str = self._get_value(str, "MarketplaceName")
        else:
            self.MarketplaceName: str = None
        if "RetrochargeTaxWithheldList" in data:
            self.RetrochargeTaxWithheldList: TaxWithheldComponentList = self._get_value(
                TaxWithheldComponentList, "RetrochargeTaxWithheldList"
            )
        else:
            self.RetrochargeTaxWithheldList: TaxWithheldComponentList = None


class SAFETReimbursementEvent(__BaseDictObject):
    """
    A SAFE-T claim reimbursement on the seller's account.
    """

    def __init__(self, data):
        super().__init__(data)
        if "PostedDate" in data:
            self.PostedDate: Date = self._get_value(Date, "PostedDate")
        else:
            self.PostedDate: Date = None
        if "SAFETClaimId" in data:
            self.SAFETClaimId: str = self._get_value(str, "SAFETClaimId")
        else:
            self.SAFETClaimId: str = None
        if "ReimbursedAmount" in data:
            self.ReimbursedAmount: Currency = self._get_value(Currency, "ReimbursedAmount")
        else:
            self.ReimbursedAmount: Currency = None
        if "ReasonCode" in data:
            self.ReasonCode: str = self._get_value(str, "ReasonCode")
        else:
            self.ReasonCode: str = None
        if "SAFETReimbursementItemList" in data:
            self.SAFETReimbursementItemList: SAFETReimbursementItemList = self._get_value(
                SAFETReimbursementItemList, "SAFETReimbursementItemList"
            )
        else:
            self.SAFETReimbursementItemList: SAFETReimbursementItemList = None


class SAFETReimbursementItem(__BaseDictObject):
    """
    An item from a SAFE-T claim reimbursement.
    """

    def __init__(self, data):
        super().__init__(data)
        if "itemChargeList" in data:
            self.itemChargeList: ChargeComponentList = self._get_value(ChargeComponentList, "itemChargeList")
        else:
            self.itemChargeList: ChargeComponentList = None
        if "productDescription" in data:
            self.productDescription: str = self._get_value(str, "productDescription")
        else:
            self.productDescription: str = None
        if "quantity" in data:
            self.quantity: str = self._get_value(str, "quantity")
        else:
            self.quantity: str = None


class SellerDealPaymentEvent(__BaseDictObject):
    """
    An event linked to the payment of a fee related to the specified deal.
    """

    def __init__(self, data):
        super().__init__(data)
        if "postedDate" in data:
            self.postedDate: Date = self._get_value(Date, "postedDate")
        else:
            self.postedDate: Date = None
        if "dealId" in data:
            self.dealId: str = self._get_value(str, "dealId")
        else:
            self.dealId: str = None
        if "dealDescription" in data:
            self.dealDescription: str = self._get_value(str, "dealDescription")
        else:
            self.dealDescription: str = None
        if "eventType" in data:
            self.eventType: str = self._get_value(str, "eventType")
        else:
            self.eventType: str = None
        if "feeType" in data:
            self.feeType: str = self._get_value(str, "feeType")
        else:
            self.feeType: str = None
        if "feeAmount" in data:
            self.feeAmount: Currency = self._get_value(Currency, "feeAmount")
        else:
            self.feeAmount: Currency = None
        if "taxAmount" in data:
            self.taxAmount: Currency = self._get_value(Currency, "taxAmount")
        else:
            self.taxAmount: Currency = None
        if "totalAmount" in data:
            self.totalAmount: Currency = self._get_value(Currency, "totalAmount")
        else:
            self.totalAmount: Currency = None


class SellerReviewEnrollmentPaymentEvent(__BaseDictObject):
    """
    A fee payment event for the Early Reviewer Program.
    """

    def __init__(self, data):
        super().__init__(data)
        if "PostedDate" in data:
            self.PostedDate: Date = self._get_value(Date, "PostedDate")
        else:
            self.PostedDate: Date = None
        if "EnrollmentId" in data:
            self.EnrollmentId: str = self._get_value(str, "EnrollmentId")
        else:
            self.EnrollmentId: str = None
        if "ParentASIN" in data:
            self.ParentASIN: str = self._get_value(str, "ParentASIN")
        else:
            self.ParentASIN: str = None
        if "FeeComponent" in data:
            self.FeeComponent: FeeComponent = self._get_value(FeeComponent, "FeeComponent")
        else:
            self.FeeComponent: FeeComponent = None
        if "ChargeComponent" in data:
            self.ChargeComponent: ChargeComponent = self._get_value(ChargeComponent, "ChargeComponent")
        else:
            self.ChargeComponent: ChargeComponent = None
        if "TotalAmount" in data:
            self.TotalAmount: Currency = self._get_value(Currency, "TotalAmount")
        else:
            self.TotalAmount: Currency = None


class ServiceFeeEvent(__BaseDictObject):
    """
    A service fee on the seller's account.
    """

    def __init__(self, data):
        super().__init__(data)
        if "AmazonOrderId" in data:
            self.AmazonOrderId: str = self._get_value(str, "AmazonOrderId")
        else:
            self.AmazonOrderId: str = None
        if "FeeReason" in data:
            self.FeeReason: str = self._get_value(str, "FeeReason")
        else:
            self.FeeReason: str = None
        if "FeeList" in data:
            self.FeeList: FeeComponentList = self._get_value(FeeComponentList, "FeeList")
        else:
            self.FeeList: FeeComponentList = None
        if "SellerSKU" in data:
            self.SellerSKU: str = self._get_value(str, "SellerSKU")
        else:
            self.SellerSKU: str = None
        if "FnSKU" in data:
            self.FnSKU: str = self._get_value(str, "FnSKU")
        else:
            self.FnSKU: str = None
        if "FeeDescription" in data:
            self.FeeDescription: str = self._get_value(str, "FeeDescription")
        else:
            self.FeeDescription: str = None
        if "ASIN" in data:
            self.ASIN: str = self._get_value(str, "ASIN")
        else:
            self.ASIN: str = None


class ShipmentEvent(__BaseDictObject):
    """
    A shipment, refund, guarantee claim, or chargeback.
    """

    def __init__(self, data):
        super().__init__(data)
        if "AmazonOrderId" in data:
            self.AmazonOrderId: str = self._get_value(str, "AmazonOrderId")
        else:
            self.AmazonOrderId: str = None
        if "SellerOrderId" in data:
            self.SellerOrderId: str = self._get_value(str, "SellerOrderId")
        else:
            self.SellerOrderId: str = None
        if "MarketplaceName" in data:
            self.MarketplaceName: str = self._get_value(str, "MarketplaceName")
        else:
            self.MarketplaceName: str = None
        if "OrderChargeList" in data:
            self.OrderChargeList: ChargeComponentList = self._get_value(ChargeComponentList, "OrderChargeList")
        else:
            self.OrderChargeList: ChargeComponentList = None
        if "OrderChargeAdjustmentList" in data:
            self.OrderChargeAdjustmentList: ChargeComponentList = self._get_value(
                ChargeComponentList, "OrderChargeAdjustmentList"
            )
        else:
            self.OrderChargeAdjustmentList: ChargeComponentList = None
        if "ShipmentFeeList" in data:
            self.ShipmentFeeList: FeeComponentList = self._get_value(FeeComponentList, "ShipmentFeeList")
        else:
            self.ShipmentFeeList: FeeComponentList = None
        if "ShipmentFeeAdjustmentList" in data:
            self.ShipmentFeeAdjustmentList: FeeComponentList = self._get_value(
                FeeComponentList, "ShipmentFeeAdjustmentList"
            )
        else:
            self.ShipmentFeeAdjustmentList: FeeComponentList = None
        if "OrderFeeList" in data:
            self.OrderFeeList: FeeComponentList = self._get_value(FeeComponentList, "OrderFeeList")
        else:
            self.OrderFeeList: FeeComponentList = None
        if "OrderFeeAdjustmentList" in data:
            self.OrderFeeAdjustmentList: FeeComponentList = self._get_value(FeeComponentList, "OrderFeeAdjustmentList")
        else:
            self.OrderFeeAdjustmentList: FeeComponentList = None
        if "DirectPaymentList" in data:
            self.DirectPaymentList: DirectPaymentList = self._get_value(DirectPaymentList, "DirectPaymentList")
        else:
            self.DirectPaymentList: DirectPaymentList = None
        if "PostedDate" in data:
            self.PostedDate: Date = self._get_value(Date, "PostedDate")
        else:
            self.PostedDate: Date = None
        if "ShipmentItemList" in data:
            self.ShipmentItemList: ShipmentItemList = self._get_value(ShipmentItemList, "ShipmentItemList")
        else:
            self.ShipmentItemList: ShipmentItemList = None
        if "ShipmentItemAdjustmentList" in data:
            self.ShipmentItemAdjustmentList: ShipmentItemList = self._get_value(
                ShipmentItemList, "ShipmentItemAdjustmentList"
            )
        else:
            self.ShipmentItemAdjustmentList: ShipmentItemList = None


class ShipmentItem(__BaseDictObject):
    """
    An item of a shipment, refund, guarantee claim, or chargeback.
    """

    def __init__(self, data):
        super().__init__(data)
        if "SellerSKU" in data:
            self.SellerSKU: str = self._get_value(str, "SellerSKU")
        else:
            self.SellerSKU: str = None
        if "OrderItemId" in data:
            self.OrderItemId: str = self._get_value(str, "OrderItemId")
        else:
            self.OrderItemId: str = None
        if "OrderAdjustmentItemId" in data:
            self.OrderAdjustmentItemId: str = self._get_value(str, "OrderAdjustmentItemId")
        else:
            self.OrderAdjustmentItemId: str = None
        if "QuantityShipped" in data:
            self.QuantityShipped: int = self._get_value(int, "QuantityShipped")
        else:
            self.QuantityShipped: int = None
        if "ItemChargeList" in data:
            self.ItemChargeList: ChargeComponentList = self._get_value(ChargeComponentList, "ItemChargeList")
        else:
            self.ItemChargeList: ChargeComponentList = None
        if "ItemChargeAdjustmentList" in data:
            self.ItemChargeAdjustmentList: ChargeComponentList = self._get_value(
                ChargeComponentList, "ItemChargeAdjustmentList"
            )
        else:
            self.ItemChargeAdjustmentList: ChargeComponentList = None
        if "ItemFeeList" in data:
            self.ItemFeeList: FeeComponentList = self._get_value(FeeComponentList, "ItemFeeList")
        else:
            self.ItemFeeList: FeeComponentList = None
        if "ItemFeeAdjustmentList" in data:
            self.ItemFeeAdjustmentList: FeeComponentList = self._get_value(FeeComponentList, "ItemFeeAdjustmentList")
        else:
            self.ItemFeeAdjustmentList: FeeComponentList = None
        if "ItemTaxWithheldList" in data:
            self.ItemTaxWithheldList: TaxWithheldComponentList = self._get_value(
                TaxWithheldComponentList, "ItemTaxWithheldList"
            )
        else:
            self.ItemTaxWithheldList: TaxWithheldComponentList = None
        if "PromotionList" in data:
            self.PromotionList: PromotionList = self._get_value(PromotionList, "PromotionList")
        else:
            self.PromotionList: PromotionList = None
        if "PromotionAdjustmentList" in data:
            self.PromotionAdjustmentList: PromotionList = self._get_value(PromotionList, "PromotionAdjustmentList")
        else:
            self.PromotionAdjustmentList: PromotionList = None
        if "CostOfPointsGranted" in data:
            self.CostOfPointsGranted: Currency = self._get_value(Currency, "CostOfPointsGranted")
        else:
            self.CostOfPointsGranted: Currency = None
        if "CostOfPointsReturned" in data:
            self.CostOfPointsReturned: Currency = self._get_value(Currency, "CostOfPointsReturned")
        else:
            self.CostOfPointsReturned: Currency = None


class SolutionProviderCreditEvent(__BaseDictObject):
    """
    A credit given to a solution provider.
    """

    def __init__(self, data):
        super().__init__(data)
        if "ProviderTransactionType" in data:
            self.ProviderTransactionType: str = self._get_value(str, "ProviderTransactionType")
        else:
            self.ProviderTransactionType: str = None
        if "SellerOrderId" in data:
            self.SellerOrderId: str = self._get_value(str, "SellerOrderId")
        else:
            self.SellerOrderId: str = None
        if "MarketplaceId" in data:
            self.MarketplaceId: str = self._get_value(str, "MarketplaceId")
        else:
            self.MarketplaceId: str = None
        if "MarketplaceCountryCode" in data:
            self.MarketplaceCountryCode: str = self._get_value(str, "MarketplaceCountryCode")
        else:
            self.MarketplaceCountryCode: str = None
        if "SellerId" in data:
            self.SellerId: str = self._get_value(str, "SellerId")
        else:
            self.SellerId: str = None
        if "SellerStoreName" in data:
            self.SellerStoreName: str = self._get_value(str, "SellerStoreName")
        else:
            self.SellerStoreName: str = None
        if "ProviderId" in data:
            self.ProviderId: str = self._get_value(str, "ProviderId")
        else:
            self.ProviderId: str = None
        if "ProviderStoreName" in data:
            self.ProviderStoreName: str = self._get_value(str, "ProviderStoreName")
        else:
            self.ProviderStoreName: str = None
        if "TransactionAmount" in data:
            self.TransactionAmount: Currency = self._get_value(Currency, "TransactionAmount")
        else:
            self.TransactionAmount: Currency = None
        if "TransactionCreationDate" in data:
            self.TransactionCreationDate: Date = self._get_value(Date, "TransactionCreationDate")
        else:
            self.TransactionCreationDate: Date = None


class TaxWithholdingPeriod(__BaseDictObject):
    """
    Period which taxwithholding on seller's account is calculated.
    """

    def __init__(self, data):
        super().__init__(data)
        if "StartDate" in data:
            self.StartDate: Date = self._get_value(Date, "StartDate")
        else:
            self.StartDate: Date = None
        if "EndDate" in data:
            self.EndDate: Date = self._get_value(Date, "EndDate")
        else:
            self.EndDate: Date = None


class TaxWithholdingEvent(__BaseDictObject):
    """
    A TaxWithholding event on seller's account.
    """

    def __init__(self, data):
        super().__init__(data)
        if "PostedDate" in data:
            self.PostedDate: Date = self._get_value(Date, "PostedDate")
        else:
            self.PostedDate: Date = None
        if "BaseAmount" in data:
            self.BaseAmount: Currency = self._get_value(Currency, "BaseAmount")
        else:
            self.BaseAmount: Currency = None
        if "WithheldAmount" in data:
            self.WithheldAmount: Currency = self._get_value(Currency, "WithheldAmount")
        else:
            self.WithheldAmount: Currency = None
        if "TaxWithholdingPeriod" in data:
            self.TaxWithholdingPeriod: TaxWithholdingPeriod = self._get_value(
                TaxWithholdingPeriod, "TaxWithholdingPeriod"
            )
        else:
            self.TaxWithholdingPeriod: TaxWithholdingPeriod = None


class TaxWithheldComponent(__BaseDictObject):
    """
    Information about the taxes withheld.
    """

    def __init__(self, data):
        super().__init__(data)
        if "TaxCollectionModel" in data:
            self.TaxCollectionModel: str = self._get_value(str, "TaxCollectionModel")
        else:
            self.TaxCollectionModel: str = None
        if "TaxesWithheld" in data:
            self.TaxesWithheld: ChargeComponentList = self._get_value(ChargeComponentList, "TaxesWithheld")
        else:
            self.TaxesWithheld: ChargeComponentList = None


class TrialShipmentEvent(__BaseDictObject):
    """
    An event related to a trial shipment.
    """

    def __init__(self, data):
        super().__init__(data)
        if "AmazonOrderId" in data:
            self.AmazonOrderId: str = self._get_value(str, "AmazonOrderId")
        else:
            self.AmazonOrderId: str = None
        if "FinancialEventGroupId" in data:
            self.FinancialEventGroupId: str = self._get_value(str, "FinancialEventGroupId")
        else:
            self.FinancialEventGroupId: str = None
        if "PostedDate" in data:
            self.PostedDate: Date = self._get_value(Date, "PostedDate")
        else:
            self.PostedDate: Date = None
        if "SKU" in data:
            self.SKU: str = self._get_value(str, "SKU")
        else:
            self.SKU: str = None
        if "FeeList" in data:
            self.FeeList: FeeComponentList = self._get_value(FeeComponentList, "FeeList")
        else:
            self.FeeList: FeeComponentList = None


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class AdjustmentEventList(list, _List["AdjustmentEvent"]):
    """
    A list of adjustment event information for the seller's account.
    """

    def __init__(self, data):
        super().__init__([AdjustmentEvent(datum) for datum in data])
        self.data = data


class AdjustmentItemList(list, _List["AdjustmentItem"]):
    """
    A list of information about items in an adjustment to the seller's account.
    """

    def __init__(self, data):
        super().__init__([AdjustmentItem(datum) for datum in data])
        self.data = data


class AffordabilityExpenseEventList(list, _List["AffordabilityExpenseEvent"]):
    """
    A list of expense information related to an affordability promotion.
    """

    def __init__(self, data):
        super().__init__([AffordabilityExpenseEvent(datum) for datum in data])
        self.data = data


class ChargeComponentList(list, _List["ChargeComponent"]):
    """
    A list of charge information on the seller's account.
    """

    def __init__(self, data):
        super().__init__([ChargeComponent(datum) for datum in data])
        self.data = data


class ChargeInstrumentList(list, _List["ChargeInstrument"]):
    """
    A list of payment instruments.
    """

    def __init__(self, data):
        super().__init__([ChargeInstrument(datum) for datum in data])
        self.data = data


class CouponPaymentEventList(list, _List["CouponPaymentEvent"]):
    """
    A list of coupon payment event information.
    """

    def __init__(self, data):
        super().__init__([CouponPaymentEvent(datum) for datum in data])
        self.data = data


class DebtRecoveryEventList(list, _List["DebtRecoveryEvent"]):
    """
    A list of debt recovery event information.
    """

    def __init__(self, data):
        super().__init__([DebtRecoveryEvent(datum) for datum in data])
        self.data = data


class DebtRecoveryItemList(list, _List["DebtRecoveryItem"]):
    """
    A list of debt recovery item information.
    """

    def __init__(self, data):
        super().__init__([DebtRecoveryItem(datum) for datum in data])
        self.data = data


class DirectPaymentList(list, _List["DirectPayment"]):
    """
    A list of direct payment information.
    """

    def __init__(self, data):
        super().__init__([DirectPayment(datum) for datum in data])
        self.data = data


class FBALiquidationEventList(list, _List["FBALiquidationEvent"]):
    """
    A list of FBA inventory liquidation payment events.
    """

    def __init__(self, data):
        super().__init__([FBALiquidationEvent(datum) for datum in data])
        self.data = data


class FeeComponentList(list, _List["FeeComponent"]):
    """
    A list of fee component information.
    """

    def __init__(self, data):
        super().__init__([FeeComponent(datum) for datum in data])
        self.data = data


class FinancialEventGroupList(list, _List["FinancialEventGroup"]):
    """
    A list of financial event group information.
    """

    def __init__(self, data):
        super().__init__([FinancialEventGroup(datum) for datum in data])
        self.data = data


class ImagingServicesFeeEventList(list, _List["ImagingServicesFeeEvent"]):
    """
    A list of fee events related to Amazon Imaging services.
    """

    def __init__(self, data):
        super().__init__([ImagingServicesFeeEvent(datum) for datum in data])
        self.data = data


class LoanServicingEventList(list, _List["LoanServicingEvent"]):
    """
    A list of loan servicing events.
    """

    def __init__(self, data):
        super().__init__([LoanServicingEvent(datum) for datum in data])
        self.data = data


class NetworkComminglingTransactionEventList(list, _List["NetworkComminglingTransactionEvent"]):
    """
    A list of network commingling transaction events.
    """

    def __init__(self, data):
        super().__init__([NetworkComminglingTransactionEvent(datum) for datum in data])
        self.data = data


class PayWithAmazonEventList(list, _List["PayWithAmazonEvent"]):
    """
    A list of events related to the seller's Pay with Amazon account.
    """

    def __init__(self, data):
        super().__init__([PayWithAmazonEvent(datum) for datum in data])
        self.data = data


class ProductAdsPaymentEventList(list, _List["ProductAdsPaymentEvent"]):
    """
    A list of sponsored products payment events.
    """

    def __init__(self, data):
        super().__init__([ProductAdsPaymentEvent(datum) for datum in data])
        self.data = data


class PromotionList(list, _List["Promotion"]):
    """
    A list of promotions.
    """

    def __init__(self, data):
        super().__init__([Promotion(datum) for datum in data])
        self.data = data


class RemovalShipmentEventList(list, _List["RemovalShipmentEvent"]):
    """
    A list of removal shipment event information.
    """

    def __init__(self, data):
        super().__init__([RemovalShipmentEvent(datum) for datum in data])
        self.data = data


class RemovalShipmentItemList(list, _List["RemovalShipmentItem"]):
    """
    A list of information about removal shipment items.
    """

    def __init__(self, data):
        super().__init__([RemovalShipmentItem(datum) for datum in data])
        self.data = data


class RemovalShipmentAdjustmentEventList(list, _List["RemovalShipmentAdjustmentEvent"]):
    """
    A comma-delimited list of Removal shipmentAdjustment details for FBA inventory.
    """

    def __init__(self, data):
        super().__init__([RemovalShipmentAdjustmentEvent(datum) for datum in data])
        self.data = data


class RentalTransactionEventList(list, _List["RentalTransactionEvent"]):
    """
    A list of rental transaction event information.
    """

    def __init__(self, data):
        super().__init__([RentalTransactionEvent(datum) for datum in data])
        self.data = data


class RetrochargeEventList(list, _List["RetrochargeEvent"]):
    """
    A list of information about Retrocharge or RetrochargeReversal events.
    """

    def __init__(self, data):
        super().__init__([RetrochargeEvent(datum) for datum in data])
        self.data = data


class SAFETReimbursementEventList(list, _List["SAFETReimbursementEvent"]):
    """
    A list of SAFETReimbursementEvents.
    """

    def __init__(self, data):
        super().__init__([SAFETReimbursementEvent(datum) for datum in data])
        self.data = data


class SAFETReimbursementItemList(list, _List["SAFETReimbursementItem"]):
    """
    A list of SAFETReimbursementItems.
    """

    def __init__(self, data):
        super().__init__([SAFETReimbursementItem(datum) for datum in data])
        self.data = data


class SellerDealPaymentEventList(list, _List["SellerDealPaymentEvent"]):
    """
    A list of payment events for deal-related fees.
    """

    def __init__(self, data):
        super().__init__([SellerDealPaymentEvent(datum) for datum in data])
        self.data = data


class SellerReviewEnrollmentPaymentEventList(list, _List["SellerReviewEnrollmentPaymentEvent"]):
    """
    A list of information about fee events for the Early Reviewer Program.
    """

    def __init__(self, data):
        super().__init__([SellerReviewEnrollmentPaymentEvent(datum) for datum in data])
        self.data = data


class ServiceFeeEventList(list, _List["ServiceFeeEvent"]):
    """
    A list of information about service fee events.
    """

    def __init__(self, data):
        super().__init__([ServiceFeeEvent(datum) for datum in data])
        self.data = data


class ShipmentEventList(list, _List["ShipmentEvent"]):
    """
    A list of shipment event information.
    """

    def __init__(self, data):
        super().__init__([ShipmentEvent(datum) for datum in data])
        self.data = data


class ShipmentItemList(list, _List["ShipmentItem"]):
    """
    A list of shipment items.
    """

    def __init__(self, data):
        super().__init__([ShipmentItem(datum) for datum in data])
        self.data = data


class SolutionProviderCreditEventList(list, _List["SolutionProviderCreditEvent"]):
    """
    A list of information about solution provider credits.
    """

    def __init__(self, data):
        super().__init__([SolutionProviderCreditEvent(datum) for datum in data])
        self.data = data


class TaxWithholdingEventList(list, _List["TaxWithholdingEvent"]):
    """
    List of TaxWithholding events.
    """

    def __init__(self, data):
        super().__init__([TaxWithholdingEvent(datum) for datum in data])
        self.data = data


class TaxWithheldComponentList(list, _List["TaxWithheldComponent"]):
    """
    A list of information about taxes withheld.
    """

    def __init__(self, data):
        super().__init__([TaxWithheldComponent(datum) for datum in data])
        self.data = data


class TrialShipmentEventList(list, _List["TrialShipmentEvent"]):
    """
    A list of information about trial shipment financial events.
    """

    def __init__(self, data):
        super().__init__([TrialShipmentEvent(datum) for datum in data])
        self.data = data


class ShipmentSettleEventList(list, _List["ShipmentEvent"]):
    """
    A list of information about shipment settle financial events.
    """

    def __init__(self, data):
        super().__init__([ShipmentEvent(datum) for datum in data])
        self.data = data


class ErrorList(list, _List["Error"]):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__([Error(datum) for datum in data])
        self.data = data


class BigDecimal(float):
    """ """


class Date(str):
    """ """


class FinancesV0Client(__BaseClient):
    def listFinancialEventGroups(
        self,
        MaxResultsPerPage: int = None,
        FinancialEventGroupStartedBefore: str = None,
        FinancialEventGroupStartedAfter: str = None,
        NextToken: str = None,
    ):
        """
                Returns financial event groups for a given date range.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 0.5 | 30 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/finances/v0/financialEventGroups"
        params = {}
        if MaxResultsPerPage is not None:
            params["MaxResultsPerPage"] = MaxResultsPerPage
        if FinancialEventGroupStartedBefore is not None:
            params["FinancialEventGroupStartedBefore"] = FinancialEventGroupStartedBefore
        if FinancialEventGroupStartedAfter is not None:
            params["FinancialEventGroupStartedAfter"] = FinancialEventGroupStartedAfter
        if NextToken is not None:
            params["NextToken"] = NextToken
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: ListFinancialEventGroupsResponse,
            400: ListFinancialEventGroupsResponse,
            403: ListFinancialEventGroupsResponse,
            404: ListFinancialEventGroupsResponse,
            429: ListFinancialEventGroupsResponse,
            500: ListFinancialEventGroupsResponse,
            503: ListFinancialEventGroupsResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def listFinancialEventsByGroupId(
        self,
        eventGroupId: str,
        MaxResultsPerPage: int = None,
        NextToken: str = None,
    ):
        """
                Returns all financial events for the specified financial event group.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 0.5 | 30 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/finances/v0/financialEventGroups/{eventGroupId}/financialEvents"
        params = {}
        if MaxResultsPerPage is not None:
            params["MaxResultsPerPage"] = MaxResultsPerPage
        if NextToken is not None:
            params["NextToken"] = NextToken
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: ListFinancialEventsResponse,
            400: ListFinancialEventsResponse,
            403: ListFinancialEventsResponse,
            404: ListFinancialEventsResponse,
            429: ListFinancialEventsResponse,
            500: ListFinancialEventsResponse,
            503: ListFinancialEventsResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def listFinancialEventsByOrderId(
        self,
        orderId: str,
        MaxResultsPerPage: int = None,
        NextToken: str = None,
    ):
        """
                Returns all financial events for the specified order.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 0.5 | 30 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/finances/v0/orders/{orderId}/financialEvents"
        params = {}
        if MaxResultsPerPage is not None:
            params["MaxResultsPerPage"] = MaxResultsPerPage
        if NextToken is not None:
            params["NextToken"] = NextToken
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: ListFinancialEventsResponse,
            400: ListFinancialEventsResponse,
            403: ListFinancialEventsResponse,
            404: ListFinancialEventsResponse,
            429: ListFinancialEventsResponse,
            500: ListFinancialEventsResponse,
            503: ListFinancialEventsResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def listFinancialEvents(
        self,
        MaxResultsPerPage: int = None,
        PostedAfter: str = None,
        PostedBefore: str = None,
        NextToken: str = None,
    ):
        """
                Returns financial events for the specified data range.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 0.5 | 30 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/finances/v0/financialEvents"
        params = {}
        if MaxResultsPerPage is not None:
            params["MaxResultsPerPage"] = MaxResultsPerPage
        if PostedAfter is not None:
            params["PostedAfter"] = PostedAfter
        if PostedBefore is not None:
            params["PostedBefore"] = PostedBefore
        if NextToken is not None:
            params["NextToken"] = NextToken
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: ListFinancialEventsResponse,
            400: ListFinancialEventsResponse,
            403: ListFinancialEventsResponse,
            404: ListFinancialEventsResponse,
            429: ListFinancialEventsResponse,
            500: ListFinancialEventsResponse,
            503: ListFinancialEventsResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))



from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class AplusResponse(__BaseDictObject):
    """
    The base response data for all A+ Content operations when a request is successful or partially successful. Individual operations may extend this with additional data.
    """

    def __init__(self, data):
        super().__init__(data)
        if "warnings" in data:
            self.warnings: MessageSet = self._get_value(MessageSet, "warnings")
        else:
            self.warnings: MessageSet = None


class ErrorList(__BaseDictObject):
    """
    The error response for when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "errors" in data:
            self.errors: _List[Error] = [Error(datum) for datum in data["errors"]]
        else:
            self.errors: _List[Error] = []


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class ContentMetadataRecord(__BaseDictObject):
    """
    The metadata for an A+ Content document, with additional information for content management.
    """

    def __init__(self, data):
        super().__init__(data)
        if "contentReferenceKey" in data:
            self.contentReferenceKey: ContentReferenceKey = self._get_value(ContentReferenceKey, "contentReferenceKey")
        else:
            self.contentReferenceKey: ContentReferenceKey = None
        if "contentMetadata" in data:
            self.contentMetadata: ContentMetadata = self._get_value(ContentMetadata, "contentMetadata")
        else:
            self.contentMetadata: ContentMetadata = None


class ContentMetadata(__BaseDictObject):
    """
    The metadata of an A+ Content document.
    """

    def __init__(self, data):
        super().__init__(data)
        if "name" in data:
            self.name: str = self._get_value(str, "name")
        else:
            self.name: str = None
        if "marketplaceId" in data:
            self.marketplaceId: MarketplaceId = self._get_value(MarketplaceId, "marketplaceId")
        else:
            self.marketplaceId: MarketplaceId = None
        if "status" in data:
            self.status: ContentStatus = self._get_value(ContentStatus, "status")
        else:
            self.status: ContentStatus = None
        if "badgeSet" in data:
            self.badgeSet: ContentBadgeSet = self._get_value(ContentBadgeSet, "badgeSet")
        else:
            self.badgeSet: ContentBadgeSet = None
        if "updateTime" in data:
            self.updateTime: str = self._get_value(str, "updateTime")
        else:
            self.updateTime: str = None


class AsinMetadata(__BaseDictObject):
    """
    The A+ Content ASIN with additional metadata for content management. If you don't include the `includedDataSet` parameter in a call to the listContentDocumentAsinRelations operation, the related ASINs are returned without metadata.
    """

    def __init__(self, data):
        super().__init__(data)
        if "asin" in data:
            self.asin: Asin = self._get_value(Asin, "asin")
        else:
            self.asin: Asin = None
        if "badgeSet" in data:
            self.badgeSet: AsinBadgeSet = self._get_value(AsinBadgeSet, "badgeSet")
        else:
            self.badgeSet: AsinBadgeSet = None
        if "parent" in data:
            self.parent: Asin = self._get_value(Asin, "parent")
        else:
            self.parent: Asin = None
        if "title" in data:
            self.title: str = self._get_value(str, "title")
        else:
            self.title: str = None
        if "imageUrl" in data:
            self.imageUrl: str = self._get_value(str, "imageUrl")
        else:
            self.imageUrl: str = None
        if "contentReferenceKeySet" in data:
            self.contentReferenceKeySet: ContentReferenceKeySet = self._get_value(
                ContentReferenceKeySet, "contentReferenceKeySet"
            )
        else:
            self.contentReferenceKeySet: ContentReferenceKeySet = None


class PublishRecord(__BaseDictObject):
    """
    The full context for an A+ Content publishing event.
    """

    def __init__(self, data):
        super().__init__(data)
        if "marketplaceId" in data:
            self.marketplaceId: MarketplaceId = self._get_value(MarketplaceId, "marketplaceId")
        else:
            self.marketplaceId: MarketplaceId = None
        if "locale" in data:
            self.locale: LanguageTag = self._get_value(LanguageTag, "locale")
        else:
            self.locale: LanguageTag = None
        if "asin" in data:
            self.asin: Asin = self._get_value(Asin, "asin")
        else:
            self.asin: Asin = None
        if "contentType" in data:
            self.contentType: ContentType = self._get_value(ContentType, "contentType")
        else:
            self.contentType: ContentType = None
        if "contentSubType" in data:
            self.contentSubType: ContentSubType = self._get_value(ContentSubType, "contentSubType")
        else:
            self.contentSubType: ContentSubType = None
        if "contentReferenceKey" in data:
            self.contentReferenceKey: ContentReferenceKey = self._get_value(ContentReferenceKey, "contentReferenceKey")
        else:
            self.contentReferenceKey: ContentReferenceKey = None


class ImageCropSpecification(__BaseDictObject):
    """
    The instructions for optionally cropping an image. If no cropping is desired, set the dimensions to the original image size. If the image is cropped and no offset values are provided, then the coordinates of the top left corner of the cropped image, relative to the original image, are defaulted to (0,0).
    """

    def __init__(self, data):
        super().__init__(data)
        if "size" in data:
            self.size: ImageDimensions = self._get_value(ImageDimensions, "size")
        else:
            self.size: ImageDimensions = None
        if "offset" in data:
            self.offset: ImageOffsets = self._get_value(ImageOffsets, "offset")
        else:
            self.offset: ImageOffsets = None


class ImageDimensions(__BaseDictObject):
    """
    The dimensions extending from the top left corner of the cropped image, or the top left corner of the original image if there is no cropping. Only `pixels` is allowed as the units value for ImageDimensions.
    """

    def __init__(self, data):
        super().__init__(data)
        if "width" in data:
            self.width: IntegerWithUnits = self._get_value(IntegerWithUnits, "width")
        else:
            self.width: IntegerWithUnits = None
        if "height" in data:
            self.height: IntegerWithUnits = self._get_value(IntegerWithUnits, "height")
        else:
            self.height: IntegerWithUnits = None


class ImageOffsets(__BaseDictObject):
    """
    The top left corner of the cropped image, specified in the original image's coordinate space.
    """

    def __init__(self, data):
        super().__init__(data)
        if "x" in data:
            self.x: IntegerWithUnits = self._get_value(IntegerWithUnits, "x")
        else:
            self.x: IntegerWithUnits = None
        if "y" in data:
            self.y: IntegerWithUnits = self._get_value(IntegerWithUnits, "y")
        else:
            self.y: IntegerWithUnits = None


class IntegerWithUnits(__BaseDictObject):
    """
    A whole number dimension and its unit of measurement. For example, this can represent 100 pixels.
    """

    def __init__(self, data):
        super().__init__(data)
        if "value" in data:
            self.value: int = self._get_value(int, "value")
        else:
            self.value: int = None
        if "units" in data:
            self.units: str = self._get_value(str, "units")
        else:
            self.units: str = None


class ContentRecord(__BaseDictObject):
    """
    A content document with additional information for content management.
    """

    def __init__(self, data):
        super().__init__(data)
        if "contentReferenceKey" in data:
            self.contentReferenceKey: ContentReferenceKey = self._get_value(ContentReferenceKey, "contentReferenceKey")
        else:
            self.contentReferenceKey: ContentReferenceKey = None
        if "contentMetadata" in data:
            self.contentMetadata: ContentMetadata = self._get_value(ContentMetadata, "contentMetadata")
        else:
            self.contentMetadata: ContentMetadata = None
        if "contentDocument" in data:
            self.contentDocument: ContentDocument = self._get_value(ContentDocument, "contentDocument")
        else:
            self.contentDocument: ContentDocument = None


class ContentDocument(__BaseDictObject):
    """
    The A+ Content document. This is the enhanced content that is published to product detail pages.
    """

    def __init__(self, data):
        super().__init__(data)
        if "name" in data:
            self.name: str = self._get_value(str, "name")
        else:
            self.name: str = None
        if "contentType" in data:
            self.contentType: ContentType = self._get_value(ContentType, "contentType")
        else:
            self.contentType: ContentType = None
        if "contentSubType" in data:
            self.contentSubType: ContentSubType = self._get_value(ContentSubType, "contentSubType")
        else:
            self.contentSubType: ContentSubType = None
        if "locale" in data:
            self.locale: LanguageTag = self._get_value(LanguageTag, "locale")
        else:
            self.locale: LanguageTag = None
        if "contentModuleList" in data:
            self.contentModuleList: ContentModuleList = self._get_value(ContentModuleList, "contentModuleList")
        else:
            self.contentModuleList: ContentModuleList = None


class ContentModule(__BaseDictObject):
    """
    An A+ Content module. An A+ Content document is composed of content modules. The contentModuleType property selects which content module types to use.
    """

    def __init__(self, data):
        super().__init__(data)
        if "contentModuleType" in data:
            self.contentModuleType: ContentModuleType = self._get_value(ContentModuleType, "contentModuleType")
        else:
            self.contentModuleType: ContentModuleType = None
        if "standardCompanyLogo" in data:
            self.standardCompanyLogo: StandardCompanyLogoModule = self._get_value(
                StandardCompanyLogoModule, "standardCompanyLogo"
            )
        else:
            self.standardCompanyLogo: StandardCompanyLogoModule = None
        if "standardComparisonTable" in data:
            self.standardComparisonTable: StandardComparisonTableModule = self._get_value(
                StandardComparisonTableModule, "standardComparisonTable"
            )
        else:
            self.standardComparisonTable: StandardComparisonTableModule = None
        if "standardFourImageText" in data:
            self.standardFourImageText: StandardFourImageTextModule = self._get_value(
                StandardFourImageTextModule, "standardFourImageText"
            )
        else:
            self.standardFourImageText: StandardFourImageTextModule = None
        if "standardFourImageTextQuadrant" in data:
            self.standardFourImageTextQuadrant: StandardFourImageTextQuadrantModule = self._get_value(
                StandardFourImageTextQuadrantModule, "standardFourImageTextQuadrant"
            )
        else:
            self.standardFourImageTextQuadrant: StandardFourImageTextQuadrantModule = None
        if "standardHeaderImageText" in data:
            self.standardHeaderImageText: StandardHeaderImageTextModule = self._get_value(
                StandardHeaderImageTextModule, "standardHeaderImageText"
            )
        else:
            self.standardHeaderImageText: StandardHeaderImageTextModule = None
        if "standardImageSidebar" in data:
            self.standardImageSidebar: StandardImageSidebarModule = self._get_value(
                StandardImageSidebarModule, "standardImageSidebar"
            )
        else:
            self.standardImageSidebar: StandardImageSidebarModule = None
        if "standardImageTextOverlay" in data:
            self.standardImageTextOverlay: StandardImageTextOverlayModule = self._get_value(
                StandardImageTextOverlayModule, "standardImageTextOverlay"
            )
        else:
            self.standardImageTextOverlay: StandardImageTextOverlayModule = None
        if "standardMultipleImageText" in data:
            self.standardMultipleImageText: StandardMultipleImageTextModule = self._get_value(
                StandardMultipleImageTextModule, "standardMultipleImageText"
            )
        else:
            self.standardMultipleImageText: StandardMultipleImageTextModule = None
        if "standardProductDescription" in data:
            self.standardProductDescription: StandardProductDescriptionModule = self._get_value(
                StandardProductDescriptionModule, "standardProductDescription"
            )
        else:
            self.standardProductDescription: StandardProductDescriptionModule = None
        if "standardSingleImageHighlights" in data:
            self.standardSingleImageHighlights: StandardSingleImageHighlightsModule = self._get_value(
                StandardSingleImageHighlightsModule, "standardSingleImageHighlights"
            )
        else:
            self.standardSingleImageHighlights: StandardSingleImageHighlightsModule = None
        if "standardSingleImageSpecsDetail" in data:
            self.standardSingleImageSpecsDetail: StandardSingleImageSpecsDetailModule = self._get_value(
                StandardSingleImageSpecsDetailModule, "standardSingleImageSpecsDetail"
            )
        else:
            self.standardSingleImageSpecsDetail: StandardSingleImageSpecsDetailModule = None
        if "standardSingleSideImage" in data:
            self.standardSingleSideImage: StandardSingleSideImageModule = self._get_value(
                StandardSingleSideImageModule, "standardSingleSideImage"
            )
        else:
            self.standardSingleSideImage: StandardSingleSideImageModule = None
        if "standardTechSpecs" in data:
            self.standardTechSpecs: StandardTechSpecsModule = self._get_value(
                StandardTechSpecsModule, "standardTechSpecs"
            )
        else:
            self.standardTechSpecs: StandardTechSpecsModule = None
        if "standardText" in data:
            self.standardText: StandardTextModule = self._get_value(StandardTextModule, "standardText")
        else:
            self.standardText: StandardTextModule = None
        if "standardThreeImageText" in data:
            self.standardThreeImageText: StandardThreeImageTextModule = self._get_value(
                StandardThreeImageTextModule, "standardThreeImageText"
            )
        else:
            self.standardThreeImageText: StandardThreeImageTextModule = None


class StandardCompanyLogoModule(__BaseDictObject):
    """
    The standard company logo image.
    """

    def __init__(self, data):
        super().__init__(data)
        if "companyLogo" in data:
            self.companyLogo: ImageComponent = self._get_value(ImageComponent, "companyLogo")
        else:
            self.companyLogo: ImageComponent = None


class StandardComparisonTableModule(__BaseDictObject):
    """
    The standard product comparison table.
    """

    def __init__(self, data):
        super().__init__(data)
        if "productColumns" in data:
            self.productColumns: _List[StandardComparisonProductBlock] = [
                StandardComparisonProductBlock(datum) for datum in data["productColumns"]
            ]
        else:
            self.productColumns: _List[StandardComparisonProductBlock] = []
        if "metricRowLabels" in data:
            self.metricRowLabels: _List[PlainTextItem] = [PlainTextItem(datum) for datum in data["metricRowLabels"]]
        else:
            self.metricRowLabels: _List[PlainTextItem] = []


class StandardFourImageTextModule(__BaseDictObject):
    """
    Four standard images with text, presented across a single row.
    """

    def __init__(self, data):
        super().__init__(data)
        if "headline" in data:
            self.headline: TextComponent = self._get_value(TextComponent, "headline")
        else:
            self.headline: TextComponent = None
        if "block1" in data:
            self.block1: StandardImageTextBlock = self._get_value(StandardImageTextBlock, "block1")
        else:
            self.block1: StandardImageTextBlock = None
        if "block2" in data:
            self.block2: StandardImageTextBlock = self._get_value(StandardImageTextBlock, "block2")
        else:
            self.block2: StandardImageTextBlock = None
        if "block3" in data:
            self.block3: StandardImageTextBlock = self._get_value(StandardImageTextBlock, "block3")
        else:
            self.block3: StandardImageTextBlock = None
        if "block4" in data:
            self.block4: StandardImageTextBlock = self._get_value(StandardImageTextBlock, "block4")
        else:
            self.block4: StandardImageTextBlock = None


class StandardFourImageTextQuadrantModule(__BaseDictObject):
    """
    Four standard images with text, presented on a grid of four quadrants.
    """

    def __init__(self, data):
        super().__init__(data)
        if "block1" in data:
            self.block1: StandardImageTextBlock = self._get_value(StandardImageTextBlock, "block1")
        else:
            self.block1: StandardImageTextBlock = None
        if "block2" in data:
            self.block2: StandardImageTextBlock = self._get_value(StandardImageTextBlock, "block2")
        else:
            self.block2: StandardImageTextBlock = None
        if "block3" in data:
            self.block3: StandardImageTextBlock = self._get_value(StandardImageTextBlock, "block3")
        else:
            self.block3: StandardImageTextBlock = None
        if "block4" in data:
            self.block4: StandardImageTextBlock = self._get_value(StandardImageTextBlock, "block4")
        else:
            self.block4: StandardImageTextBlock = None


class StandardHeaderImageTextModule(__BaseDictObject):
    """
    Standard headline text, an image, and body text.
    """

    def __init__(self, data):
        super().__init__(data)
        if "headline" in data:
            self.headline: TextComponent = self._get_value(TextComponent, "headline")
        else:
            self.headline: TextComponent = None
        if "block" in data:
            self.block: StandardImageTextBlock = self._get_value(StandardImageTextBlock, "block")
        else:
            self.block: StandardImageTextBlock = None


class StandardImageSidebarModule(__BaseDictObject):
    """
    Two images, two paragraphs, and two bulleted lists. One image is smaller and displayed in the sidebar.
    """

    def __init__(self, data):
        super().__init__(data)
        if "headline" in data:
            self.headline: TextComponent = self._get_value(TextComponent, "headline")
        else:
            self.headline: TextComponent = None
        if "imageCaptionBlock" in data:
            self.imageCaptionBlock: StandardImageCaptionBlock = self._get_value(
                StandardImageCaptionBlock, "imageCaptionBlock"
            )
        else:
            self.imageCaptionBlock: StandardImageCaptionBlock = None
        if "descriptionTextBlock" in data:
            self.descriptionTextBlock: StandardTextBlock = self._get_value(StandardTextBlock, "descriptionTextBlock")
        else:
            self.descriptionTextBlock: StandardTextBlock = None
        if "descriptionListBlock" in data:
            self.descriptionListBlock: StandardTextListBlock = self._get_value(
                StandardTextListBlock, "descriptionListBlock"
            )
        else:
            self.descriptionListBlock: StandardTextListBlock = None
        if "sidebarImageTextBlock" in data:
            self.sidebarImageTextBlock: StandardImageTextBlock = self._get_value(
                StandardImageTextBlock, "sidebarImageTextBlock"
            )
        else:
            self.sidebarImageTextBlock: StandardImageTextBlock = None
        if "sidebarListBlock" in data:
            self.sidebarListBlock: StandardTextListBlock = self._get_value(StandardTextListBlock, "sidebarListBlock")
        else:
            self.sidebarListBlock: StandardTextListBlock = None


class StandardImageTextOverlayModule(__BaseDictObject):
    """
    A standard background image with a floating text box.
    """

    def __init__(self, data):
        super().__init__(data)
        if "overlayColorType" in data:
            self.overlayColorType: ColorType = self._get_value(ColorType, "overlayColorType")
        else:
            self.overlayColorType: ColorType = None
        if "block" in data:
            self.block: StandardImageTextBlock = self._get_value(StandardImageTextBlock, "block")
        else:
            self.block: StandardImageTextBlock = None


class StandardMultipleImageTextModule(__BaseDictObject):
    """
    Standard images with text, presented one at a time. The user clicks on thumbnails to view each block.
    """

    def __init__(self, data):
        super().__init__(data)
        if "blocks" in data:
            self.blocks: _List[StandardImageTextCaptionBlock] = [
                StandardImageTextCaptionBlock(datum) for datum in data["blocks"]
            ]
        else:
            self.blocks: _List[StandardImageTextCaptionBlock] = []


class StandardProductDescriptionModule(__BaseDictObject):
    """
    Standard product description text.
    """

    def __init__(self, data):
        super().__init__(data)
        if "body" in data:
            self.body: ParagraphComponent = self._get_value(ParagraphComponent, "body")
        else:
            self.body: ParagraphComponent = None


class StandardSingleImageHighlightsModule(__BaseDictObject):
    """
    A standard image with several paragraphs and a bulleted list.
    """

    def __init__(self, data):
        super().__init__(data)
        if "image" in data:
            self.image: ImageComponent = self._get_value(ImageComponent, "image")
        else:
            self.image: ImageComponent = None
        if "headline" in data:
            self.headline: TextComponent = self._get_value(TextComponent, "headline")
        else:
            self.headline: TextComponent = None
        if "textBlock1" in data:
            self.textBlock1: StandardTextBlock = self._get_value(StandardTextBlock, "textBlock1")
        else:
            self.textBlock1: StandardTextBlock = None
        if "textBlock2" in data:
            self.textBlock2: StandardTextBlock = self._get_value(StandardTextBlock, "textBlock2")
        else:
            self.textBlock2: StandardTextBlock = None
        if "textBlock3" in data:
            self.textBlock3: StandardTextBlock = self._get_value(StandardTextBlock, "textBlock3")
        else:
            self.textBlock3: StandardTextBlock = None
        if "bulletedListBlock" in data:
            self.bulletedListBlock: StandardHeaderTextListBlock = self._get_value(
                StandardHeaderTextListBlock, "bulletedListBlock"
            )
        else:
            self.bulletedListBlock: StandardHeaderTextListBlock = None


class StandardSingleImageSpecsDetailModule(__BaseDictObject):
    """
    A standard image with paragraphs and a bulleted list, and extra space for technical details.
    """

    def __init__(self, data):
        super().__init__(data)
        if "headline" in data:
            self.headline: TextComponent = self._get_value(TextComponent, "headline")
        else:
            self.headline: TextComponent = None
        if "image" in data:
            self.image: ImageComponent = self._get_value(ImageComponent, "image")
        else:
            self.image: ImageComponent = None
        if "descriptionHeadline" in data:
            self.descriptionHeadline: TextComponent = self._get_value(TextComponent, "descriptionHeadline")
        else:
            self.descriptionHeadline: TextComponent = None
        if "descriptionBlock1" in data:
            self.descriptionBlock1: StandardTextBlock = self._get_value(StandardTextBlock, "descriptionBlock1")
        else:
            self.descriptionBlock1: StandardTextBlock = None
        if "descriptionBlock2" in data:
            self.descriptionBlock2: StandardTextBlock = self._get_value(StandardTextBlock, "descriptionBlock2")
        else:
            self.descriptionBlock2: StandardTextBlock = None
        if "specificationHeadline" in data:
            self.specificationHeadline: TextComponent = self._get_value(TextComponent, "specificationHeadline")
        else:
            self.specificationHeadline: TextComponent = None
        if "specificationListBlock" in data:
            self.specificationListBlock: StandardHeaderTextListBlock = self._get_value(
                StandardHeaderTextListBlock, "specificationListBlock"
            )
        else:
            self.specificationListBlock: StandardHeaderTextListBlock = None
        if "specificationTextBlock" in data:
            self.specificationTextBlock: StandardTextBlock = self._get_value(
                StandardTextBlock, "specificationTextBlock"
            )
        else:
            self.specificationTextBlock: StandardTextBlock = None


class StandardSingleSideImageModule(__BaseDictObject):
    """
    A standard headline and body text with an image on the side.
    """

    def __init__(self, data):
        super().__init__(data)
        if "imagePositionType" in data:
            self.imagePositionType: PositionType = self._get_value(PositionType, "imagePositionType")
        else:
            self.imagePositionType: PositionType = None
        if "block" in data:
            self.block: StandardImageTextBlock = self._get_value(StandardImageTextBlock, "block")
        else:
            self.block: StandardImageTextBlock = None


class StandardTechSpecsModule(__BaseDictObject):
    """
    The standard table of technical feature names and definitions.
    """

    def __init__(self, data):
        super().__init__(data)
        if "headline" in data:
            self.headline: TextComponent = self._get_value(TextComponent, "headline")
        else:
            self.headline: TextComponent = None
        if "specificationList" in data:
            self.specificationList: _List[StandardTextPairBlock] = [
                StandardTextPairBlock(datum) for datum in data["specificationList"]
            ]
        else:
            self.specificationList: _List[StandardTextPairBlock] = []
        if "tableCount" in data:
            self.tableCount: int = self._get_value(int, "tableCount")
        else:
            self.tableCount: int = None


class StandardTextModule(__BaseDictObject):
    """
    A standard headline and body text.
    """

    def __init__(self, data):
        super().__init__(data)
        if "headline" in data:
            self.headline: TextComponent = self._get_value(TextComponent, "headline")
        else:
            self.headline: TextComponent = None
        if "body" in data:
            self.body: ParagraphComponent = self._get_value(ParagraphComponent, "body")
        else:
            self.body: ParagraphComponent = None


class StandardThreeImageTextModule(__BaseDictObject):
    """
    Three standard images with text, presented across a single row.
    """

    def __init__(self, data):
        super().__init__(data)
        if "headline" in data:
            self.headline: TextComponent = self._get_value(TextComponent, "headline")
        else:
            self.headline: TextComponent = None
        if "block1" in data:
            self.block1: StandardImageTextBlock = self._get_value(StandardImageTextBlock, "block1")
        else:
            self.block1: StandardImageTextBlock = None
        if "block2" in data:
            self.block2: StandardImageTextBlock = self._get_value(StandardImageTextBlock, "block2")
        else:
            self.block2: StandardImageTextBlock = None
        if "block3" in data:
            self.block3: StandardImageTextBlock = self._get_value(StandardImageTextBlock, "block3")
        else:
            self.block3: StandardImageTextBlock = None


class StandardComparisonProductBlock(__BaseDictObject):
    """
    The A+ Content standard comparison product block.
    """

    def __init__(self, data):
        super().__init__(data)
        if "position" in data:
            self.position: int = self._get_value(int, "position")
        else:
            self.position: int = None
        if "image" in data:
            self.image: ImageComponent = self._get_value(ImageComponent, "image")
        else:
            self.image: ImageComponent = None
        if "title" in data:
            self.title: str = self._get_value(str, "title")
        else:
            self.title: str = None
        if "asin" in data:
            self.asin: Asin = self._get_value(Asin, "asin")
        else:
            self.asin: Asin = None
        if "highlight" in data:
            self.highlight: bool = self._get_value(convert_bool, "highlight")
        else:
            self.highlight: bool = None
        if "metrics" in data:
            self.metrics: _List[PlainTextItem] = [PlainTextItem(datum) for datum in data["metrics"]]
        else:
            self.metrics: _List[PlainTextItem] = []


class StandardHeaderTextListBlock(__BaseDictObject):
    """
    The A+ standard fixed-length list of text, with a related headline.
    """

    def __init__(self, data):
        super().__init__(data)
        if "headline" in data:
            self.headline: TextComponent = self._get_value(TextComponent, "headline")
        else:
            self.headline: TextComponent = None
        if "block" in data:
            self.block: StandardTextListBlock = self._get_value(StandardTextListBlock, "block")
        else:
            self.block: StandardTextListBlock = None


class StandardTextListBlock(__BaseDictObject):
    """
    The A+ Content standard fixed length list of text, usually presented as bullet points.
    """

    def __init__(self, data):
        super().__init__(data)
        if "textList" in data:
            self.textList: _List[TextItem] = [TextItem(datum) for datum in data["textList"]]
        else:
            self.textList: _List[TextItem] = []


class StandardImageTextCaptionBlock(__BaseDictObject):
    """
    The A+ Content standard image and text block, with a related caption. The caption may not display on all devices.
    """

    def __init__(self, data):
        super().__init__(data)
        if "block" in data:
            self.block: StandardImageTextBlock = self._get_value(StandardImageTextBlock, "block")
        else:
            self.block: StandardImageTextBlock = None
        if "caption" in data:
            self.caption: TextComponent = self._get_value(TextComponent, "caption")
        else:
            self.caption: TextComponent = None


class StandardImageCaptionBlock(__BaseDictObject):
    """
    The A+ Content standard image and caption block.
    """

    def __init__(self, data):
        super().__init__(data)
        if "image" in data:
            self.image: ImageComponent = self._get_value(ImageComponent, "image")
        else:
            self.image: ImageComponent = None
        if "caption" in data:
            self.caption: TextComponent = self._get_value(TextComponent, "caption")
        else:
            self.caption: TextComponent = None


class StandardImageTextBlock(__BaseDictObject):
    """
    The A+ Content standard image and text box block.
    """

    def __init__(self, data):
        super().__init__(data)
        if "image" in data:
            self.image: ImageComponent = self._get_value(ImageComponent, "image")
        else:
            self.image: ImageComponent = None
        if "headline" in data:
            self.headline: TextComponent = self._get_value(TextComponent, "headline")
        else:
            self.headline: TextComponent = None
        if "body" in data:
            self.body: ParagraphComponent = self._get_value(ParagraphComponent, "body")
        else:
            self.body: ParagraphComponent = None


class StandardTextBlock(__BaseDictObject):
    """
    The A+ Content standard text box block, comprised of a paragraph with a headline.
    """

    def __init__(self, data):
        super().__init__(data)
        if "headline" in data:
            self.headline: TextComponent = self._get_value(TextComponent, "headline")
        else:
            self.headline: TextComponent = None
        if "body" in data:
            self.body: ParagraphComponent = self._get_value(ParagraphComponent, "body")
        else:
            self.body: ParagraphComponent = None


class StandardTextPairBlock(__BaseDictObject):
    """
    The A+ Content standard label and description block, comprised of a pair of text components.
    """

    def __init__(self, data):
        super().__init__(data)
        if "label" in data:
            self.label: TextComponent = self._get_value(TextComponent, "label")
        else:
            self.label: TextComponent = None
        if "description" in data:
            self.description: TextComponent = self._get_value(TextComponent, "description")
        else:
            self.description: TextComponent = None


class TextItem(__BaseDictObject):
    """
    Rich positional text, usually presented as a collection of bullet points.
    """

    def __init__(self, data):
        super().__init__(data)
        if "position" in data:
            self.position: int = self._get_value(int, "position")
        else:
            self.position: int = None
        if "text" in data:
            self.text: TextComponent = self._get_value(TextComponent, "text")
        else:
            self.text: TextComponent = None


class PlainTextItem(__BaseDictObject):
    """
    Plain positional text, used in collections of brief labels and descriptors.
    """

    def __init__(self, data):
        super().__init__(data)
        if "position" in data:
            self.position: int = self._get_value(int, "position")
        else:
            self.position: int = None
        if "value" in data:
            self.value: str = self._get_value(str, "value")
        else:
            self.value: str = None


class ImageComponent(__BaseDictObject):
    """
    A reference to an image, hosted in the A+ Content media library.
    """

    def __init__(self, data):
        super().__init__(data)
        if "uploadDestinationId" in data:
            self.uploadDestinationId: str = self._get_value(str, "uploadDestinationId")
        else:
            self.uploadDestinationId: str = None
        if "imageCropSpecification" in data:
            self.imageCropSpecification: ImageCropSpecification = self._get_value(
                ImageCropSpecification, "imageCropSpecification"
            )
        else:
            self.imageCropSpecification: ImageCropSpecification = None
        if "altText" in data:
            self.altText: str = self._get_value(str, "altText")
        else:
            self.altText: str = None


class ParagraphComponent(__BaseDictObject):
    """
    A list of rich text content, usually presented in a text box.
    """

    def __init__(self, data):
        super().__init__(data)
        if "textList" in data:
            self.textList: _List[TextComponent] = [TextComponent(datum) for datum in data["textList"]]
        else:
            self.textList: _List[TextComponent] = []


class TextComponent(__BaseDictObject):
    """
    Rich text content.
    """

    def __init__(self, data):
        super().__init__(data)
        if "value" in data:
            self.value: str = self._get_value(str, "value")
        else:
            self.value: str = None
        if "decoratorSet" in data:
            self.decoratorSet: DecoratorSet = self._get_value(DecoratorSet, "decoratorSet")
        else:
            self.decoratorSet: DecoratorSet = None


class Decorator(__BaseDictObject):
    """
    A decorator applied to a content string value in order to create rich text.
    """

    def __init__(self, data):
        super().__init__(data)
        if "type" in data:
            self.type: DecoratorType = self._get_value(DecoratorType, "type")
        else:
            self.type: DecoratorType = None
        if "offset" in data:
            self.offset: int = self._get_value(int, "offset")
        else:
            self.offset: int = None
        if "length" in data:
            self.length: int = self._get_value(int, "length")
        else:
            self.length: int = None
        if "depth" in data:
            self.depth: int = self._get_value(int, "depth")
        else:
            self.depth: int = None


class PostContentDocumentRequest(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "contentDocument" in data:
            self.contentDocument: ContentDocument = self._get_value(ContentDocument, "contentDocument")
        else:
            self.contentDocument: ContentDocument = None


class PostContentDocumentAsinRelationsRequest(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "asinSet" in data:
            self.asinSet: AsinSet = self._get_value(AsinSet, "asinSet")
        else:
            self.asinSet: AsinSet = None


class MessageSet(list, _List["Error"]):
    """
    A set of messages to the user, such as warnings or comments.
    """

    def __init__(self, data):
        super().__init__([Error(datum) for datum in data])
        self.data = data


class ContentMetadataRecordList(list, _List["ContentMetadataRecord"]):
    """
    A list of A+ Content metadata records.
    """

    def __init__(self, data):
        super().__init__([ContentMetadataRecord(datum) for datum in data])
        self.data = data


class ContentBadgeSet(list, _List["ContentBadge"]):
    """
    The set of content badges.
    """

    def __init__(self, data):
        super().__init__([ContentBadge(datum) for datum in data])
        self.data = data


class AsinBadgeSet(list, _List["AsinBadge"]):
    """
    The set of ASIN badges.
    """

    def __init__(self, data):
        super().__init__([AsinBadge(datum) for datum in data])
        self.data = data


class AsinSet(list, _List["Asin"]):
    """
    The set of ASINs.
    """

    def __init__(self, data):
        super().__init__([Asin(datum) for datum in data])
        self.data = data


class AsinMetadataSet(list, _List["AsinMetadata"]):
    """
    The set of ASIN metadata.
    """

    def __init__(self, data):
        super().__init__([AsinMetadata(datum) for datum in data])
        self.data = data


class PublishRecordList(list, _List["PublishRecord"]):
    """
    A list of A+ Content publishing records.
    """

    def __init__(self, data):
        super().__init__([PublishRecord(datum) for datum in data])
        self.data = data


class ContentReferenceKeySet(list, _List["ContentReferenceKey"]):
    """
    A set of content reference keys.
    """

    def __init__(self, data):
        super().__init__([ContentReferenceKey(datum) for datum in data])
        self.data = data


class ContentModuleList(list, _List["ContentModule"]):
    """
    A list of A+ Content modules.
    """

    def __init__(self, data):
        super().__init__([ContentModule(datum) for datum in data])
        self.data = data


class DecoratorSet(list, _List["Decorator"]):
    """
    A set of content decorators.
    """

    def __init__(self, data):
        super().__init__([Decorator(datum) for datum in data])
        self.data = data


class ContentType(str):
    """
    The A+ Content document type.
    """


class ContentSubType(str):
    """
    The A+ Content document subtype. This represents a special-purpose type of an A+ Content document. Not every A+ Content document type will have a subtype, and subtypes may change at any time.
    """


class ContentStatus(str):
    """
    The submission status of the content document.
    """


class ContentBadge(str):
    """
    A flag that provides additional information about an A+ Content document.
    """


class AsinBadge(str):
    """
    A flag that provides additional information about an ASIN. This is contextual and may change depending on the request that generated it.
    """


class MarketplaceId(str):
    """
    The identifier for the marketplace where the A+ Content is published.
    """


class LanguageTag(str):
    """
        The IETF language tag. This only supports the primary language subtag with one secondary language subtag. The secondary language subtag is almost always a regional designation. This does not support additional subtags beyond the primary and secondary subtags.
    **Pattern:** ^[a-z]{2,}-[A-Z0-9]{2,}$
    """


class Asin(str):
    """
    The Amazon Standard Identification Number (ASIN).
    """


class ContentReferenceKey(str):
    """
    A unique reference key for the A+ Content document. A content reference key cannot form a permalink and may change in the future. A content reference key is not guaranteed to match any A+ content identifier.
    """


class PageToken(str):
    """
    A page token that is returned when the results of the call exceed the page size. To get another page of results, call the operation again, passing in this value with the pageToken parameter.
    """


class ContentModuleType(str):
    """
    The type of A+ Content module.
    """


class ColorType(str):
    """
    The relative color scheme of content.
    """


class PositionType(str):
    """
    The relative positioning of content.
    """


class DecoratorType(str):
    """
    The type of rich text decorator.
    """


class contentReferenceKey(str):
    """
    The unique reference key for the A+ Content document. A content reference key cannot form a permalink and may change in the future. A content reference key is not guaranteed to match any A+ content identifier.
    """


class marketplaceId(str):
    """
    The identifier for the marketplace where the A+ Content is published.
    """


class pageToken(str):
    """
    A page token from the nextPageToken response element returned by your previous call to this operation. nextPageToken is returned when the results of a call exceed the page size. To get the next page of results, call the operation and include pageToken as the only parameter. Specifying pageToken with any other parameter will cause the request to fail. When no nextPageToken value is returned there are no more pages to return. A pageToken value is not usable across different operations.
    """


class asinSet(list):
    """
    The set of ASINs.
    """


class asin(str):
    """
    The Amazon Standard Identification Number (ASIN).
    """


class getContentDocumentIncludedDataSet(list):
    """
    The set of A+ data types to include in the response.
    """


class listContentDocumentAsinRelationsIncludedDataSet(list):
    """
    The set of A+ data types to include in the response.
    """


class AplusPaginatedResponse(
    AplusResponse,
):
    """
    The base response data for paginated A+ Content operations. Individual operations may extend this with additional data. If nextPageToken is not returned, there are no more pages to return.
    """

    def __init__(self, data):
        if "nextPageToken" in data:
            self.nextPageToken: PageToken = PageToken(data.pop("nextPageToken"))
        else:
            self.nextPageToken: PageToken = None
        self.data = data
        super().__init__(data)


class SearchContentDocumentsResponse(
    AplusPaginatedResponse,
):
    """ """

    def __init__(self, data):
        if "contentMetadataRecords" in data:
            self.contentMetadataRecords: ContentMetadataRecordList = ContentMetadataRecordList(
                data.pop("contentMetadataRecords")
            )
        else:
            self.contentMetadataRecords: ContentMetadataRecordList = None
        self.data = data
        super().__init__(data)


class GetContentDocumentResponse(
    AplusResponse,
):
    """ """

    def __init__(self, data):
        if "contentRecord" in data:
            self.contentRecord: ContentRecord = ContentRecord(data.pop("contentRecord"))
        else:
            self.contentRecord: ContentRecord = None
        self.data = data
        super().__init__(data)


class PostContentDocumentResponse(
    AplusResponse,
):
    """ """

    def __init__(self, data):
        if "contentReferenceKey" in data:
            self.contentReferenceKey: ContentReferenceKey = ContentReferenceKey(data.pop("contentReferenceKey"))
        else:
            self.contentReferenceKey: ContentReferenceKey = None
        self.data = data
        super().__init__(data)


class ListContentDocumentAsinRelationsResponse(
    AplusPaginatedResponse,
):
    """ """

    def __init__(self, data):
        if "asinMetadataSet" in data:
            self.asinMetadataSet: AsinMetadataSet = AsinMetadataSet(data.pop("asinMetadataSet"))
        else:
            self.asinMetadataSet: AsinMetadataSet = None
        self.data = data
        super().__init__(data)


class PostContentDocumentAsinRelationsResponse(
    AplusResponse,
):
    """ """

    def __init__(self, data):
        self.data = data
        super().__init__(data)


class ValidateContentDocumentAsinRelationsResponse(
    AplusResponse,
    ErrorList,
):
    """ """

    def __init__(self, data):
        self.data = data
        super().__init__(data)


class SearchContentPublishRecordsResponse(
    AplusPaginatedResponse,
):
    """ """

    def __init__(self, data):
        if "publishRecordList" in data:
            self.publishRecordList: PublishRecordList = PublishRecordList(data.pop("publishRecordList"))
        else:
            self.publishRecordList: PublishRecordList = None
        self.data = data
        super().__init__(data)


class PostContentDocumentApprovalSubmissionResponse(
    AplusResponse,
):
    """ """

    def __init__(self, data):
        self.data = data
        super().__init__(data)


class PostContentDocumentSuspendSubmissionResponse(
    AplusResponse,
):
    """ """

    def __init__(self, data):
        self.data = data
        super().__init__(data)


class AplusContent20201101Client(__BaseClient):
    def searchContentDocuments(
        self,
        marketplaceId: str,
        pageToken: str = None,
    ):
        """
                Returns a list of all A+ Content documents assigned to a selling partner. This operation returns only the metadata of the A+ Content documents. Call the getContentDocument operation to get the actual contents of the A+ Content documents.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 10 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/aplus/2020-11-01/contentDocuments"
        params = {}
        if marketplaceId is not None:
            params["marketplaceId"] = marketplaceId
        if pageToken is not None:
            params["pageToken"] = pageToken
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: SearchContentDocumentsResponse,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            410: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def createContentDocument(
        self,
        data: PostContentDocumentRequest,
        marketplaceId: str,
    ):
        """
                Creates a new A+ Content document.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 10 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/aplus/2020-11-01/contentDocuments"
        params = {}
        if marketplaceId is not None:
            params["marketplaceId"] = marketplaceId
        response = self.request(
            path=url,
            method="POST",
            params=params,
            data=data.data,
        )
        response_type = {
            200: PostContentDocumentResponse,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getContentDocument(
        self,
        contentReferenceKey: str,
        marketplaceId: str,
        includedDataSet: _List[str],
    ):
        """
                Returns an A+ Content document, if available.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 10 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/aplus/2020-11-01/contentDocuments/{contentReferenceKey}"
        params = {}
        if marketplaceId is not None:
            params["marketplaceId"] = marketplaceId
        if includedDataSet is not None:
            params["includedDataSet"] = ",".join(map(str, includedDataSet))
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetContentDocumentResponse,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            410: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def updateContentDocument(
        self,
        data: PostContentDocumentRequest,
        contentReferenceKey: str,
        marketplaceId: str,
    ):
        """
                Updates an existing A+ Content document.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 10 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/aplus/2020-11-01/contentDocuments/{contentReferenceKey}"
        params = {}
        if marketplaceId is not None:
            params["marketplaceId"] = marketplaceId
        response = self.request(
            path=url,
            method="POST",
            params=params,
            data=data.data,
        )
        response_type = {
            200: PostContentDocumentResponse,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            410: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def listContentDocumentAsinRelations(
        self,
        contentReferenceKey: str,
        marketplaceId: str,
        includedDataSet: _List[str] = None,
        asinSet: _List[str] = None,
        pageToken: str = None,
    ):
        """
                Returns a list of ASINs related to the specified A+ Content document, if available. If you do not include the asinSet parameter, the operation returns all ASINs related to the content document.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 10 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/aplus/2020-11-01/contentDocuments/{contentReferenceKey}/asins"
        params = {}
        if marketplaceId is not None:
            params["marketplaceId"] = marketplaceId
        if includedDataSet is not None:
            params["includedDataSet"] = ",".join(map(str, includedDataSet))
        if asinSet is not None:
            params["asinSet"] = ",".join(map(str, asinSet))
        if pageToken is not None:
            params["pageToken"] = pageToken
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: ListContentDocumentAsinRelationsResponse,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            410: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def postContentDocumentAsinRelations(
        self,
        data: PostContentDocumentAsinRelationsRequest,
        contentReferenceKey: str,
        marketplaceId: str,
    ):
        """
                Replaces all ASINs related to the specified A+ Content document, if available. This may add or remove ASINs, depending on the current set of related ASINs. Removing an ASIN has the side effect of suspending the content document from that ASIN.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 10 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/aplus/2020-11-01/contentDocuments/{contentReferenceKey}/asins"
        params = {}
        if marketplaceId is not None:
            params["marketplaceId"] = marketplaceId
        response = self.request(
            path=url,
            method="POST",
            params=params,
            data=data.data,
        )
        response_type = {
            200: PostContentDocumentAsinRelationsResponse,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            410: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def validateContentDocumentAsinRelations(
        self,
        data: PostContentDocumentRequest,
        marketplaceId: str,
        asinSet: _List[str] = None,
    ):
        """
                Checks if the A+ Content document is valid for use on a set of ASINs.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 10 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/aplus/2020-11-01/contentAsinValidations"
        params = {}
        if marketplaceId is not None:
            params["marketplaceId"] = marketplaceId
        if asinSet is not None:
            params["asinSet"] = ",".join(map(str, asinSet))
        response = self.request(
            path=url,
            method="POST",
            params=params,
            data=data.data,
        )
        response_type = {
            200: ValidateContentDocumentAsinRelationsResponse,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def searchContentPublishRecords(
        self,
        marketplaceId: str,
        asin: str,
        pageToken: str = None,
    ):
        """
                Searches for A+ Content publishing records, if available.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 10 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/aplus/2020-11-01/contentPublishRecords"
        params = {}
        if marketplaceId is not None:
            params["marketplaceId"] = marketplaceId
        if asin is not None:
            params["asin"] = asin
        if pageToken is not None:
            params["pageToken"] = pageToken
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: SearchContentPublishRecordsResponse,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def postContentDocumentApprovalSubmission(
        self,
        contentReferenceKey: str,
        marketplaceId: str,
    ):
        """
                Submits an A+ Content document for review, approval, and publishing.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 10 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/aplus/2020-11-01/contentDocuments/{contentReferenceKey}/approvalSubmissions"
        params = {}
        if marketplaceId is not None:
            params["marketplaceId"] = marketplaceId
        response = self.request(
            path=url,
            method="POST",
            params=params,
        )
        response_type = {
            200: PostContentDocumentApprovalSubmissionResponse,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            410: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def postContentDocumentSuspendSubmission(
        self,
        contentReferenceKey: str,
        marketplaceId: str,
    ):
        """
                Submits a request to suspend visible A+ Content. This neither deletes the content document nor the ASIN relations.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 10 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/aplus/2020-11-01/contentDocuments/{contentReferenceKey}/suspendSubmissions"
        params = {}
        if marketplaceId is not None:
            params["marketplaceId"] = marketplaceId
        response = self.request(
            path=url,
            method="POST",
            params=params,
        )
        response_type = {
            200: PostContentDocumentSuspendSubmissionResponse,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            410: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))



from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class LinkObject(__BaseDictObject):
    """
    A Link object.
    """

    def __init__(self, data):
        super().__init__(data)
        if "href" in data:
            self.href: str = self._get_value(str, "href")
        else:
            self.href: str = None
        if "name" in data:
            self.name: str = self._get_value(str, "name")
        else:
            self.name: str = None


class SolicitationsAction(__BaseDictObject):
    """
    A simple object containing the name of the template.
    """

    def __init__(self, data):
        super().__init__(data)
        if "name" in data:
            self.name: str = self._get_value(str, "name")
        else:
            self.name: str = None


class Schema(__BaseDictObject):
    """
    A JSON schema document describing the expected payload of the action. This object can be validated against <a href=http://json-schema.org/draft-04/schema>http://json-schema.org/draft-04/schema</a>.
    """

    def __init__(self, data):
        super().__init__(data)


class GetSolicitationActionsForOrderResponse(__BaseDictObject):
    """
    The response schema for the getSolicitationActionsForOrder operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "_links" in data:
            self._links: dict = self._get_value(dict, "_links")
        else:
            self._links: dict = None
        if "_embedded" in data:
            self._embedded: dict = self._get_value(dict, "_embedded")
        else:
            self._embedded: dict = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class GetSolicitationActionResponse(__BaseDictObject):
    """
    Describes a solicitation action that can be taken for an order. Provides a JSON Hypertext Application Language (HAL) link to the JSON schema document that describes the expected input.
    """

    def __init__(self, data):
        super().__init__(data)
        if "_links" in data:
            self._links: dict = self._get_value(dict, "_links")
        else:
            self._links: dict = None
        if "_embedded" in data:
            self._embedded: dict = self._get_value(dict, "_embedded")
        else:
            self._embedded: dict = None
        if "payload" in data:
            self.payload: SolicitationsAction = self._get_value(SolicitationsAction, "payload")
        else:
            self.payload: SolicitationsAction = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class GetSchemaResponse(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "_links" in data:
            self._links: dict = self._get_value(dict, "_links")
        else:
            self._links: dict = None
        if "payload" in data:
            self.payload: Schema = self._get_value(Schema, "payload")
        else:
            self.payload: Schema = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class CreateProductReviewAndSellerFeedbackSolicitationResponse(__BaseDictObject):
    """
    The response schema for the createProductReviewAndSellerFeedbackSolicitation operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class ErrorList(list, _List["Error"]):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__([Error(datum) for datum in data])
        self.data = data


class SolicitationsV1Client(__BaseClient):
    def getSolicitationActionsForOrder(
        self,
        amazonOrderId: str,
        marketplaceIds: _List[str],
    ):
        """
                Returns a list of solicitation types that are available for an order that you specify. A solicitation type is represented by an actions object, which contains a path and query parameter(s). You can use the path and parameter(s) to call an operation that sends a solicitation. Currently only the productReviewAndSellerFeedbackSolicitation solicitation type is available.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 1 | 5 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/solicitations/v1/orders/{amazonOrderId}"
        params = {}
        if marketplaceIds is not None:
            params["marketplaceIds"] = ",".join(map(str, marketplaceIds))
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetSolicitationActionsForOrderResponse,
            400: GetSolicitationActionsForOrderResponse,
            403: GetSolicitationActionsForOrderResponse,
            404: GetSolicitationActionsForOrderResponse,
            413: GetSolicitationActionsForOrderResponse,
            415: GetSolicitationActionsForOrderResponse,
            429: GetSolicitationActionsForOrderResponse,
            500: GetSolicitationActionsForOrderResponse,
            503: GetSolicitationActionsForOrderResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def createProductReviewAndSellerFeedbackSolicitation(
        self,
        amazonOrderId: str,
        marketplaceIds: _List[str],
    ):
        """
                Sends a solicitation to a buyer asking for seller feedback and a product review for the specified order. Send only one productReviewAndSellerFeedback or free form proactive message per order.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 1 | 5 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/solicitations/v1/orders/{amazonOrderId}/solicitations/productReviewAndSellerFeedback"
        params = {}
        if marketplaceIds is not None:
            params["marketplaceIds"] = ",".join(map(str, marketplaceIds))
        response = self.request(
            path=url,
            method="POST",
            params=params,
        )
        response_type = {
            201: CreateProductReviewAndSellerFeedbackSolicitationResponse,
            400: CreateProductReviewAndSellerFeedbackSolicitationResponse,
            403: CreateProductReviewAndSellerFeedbackSolicitationResponse,
            404: CreateProductReviewAndSellerFeedbackSolicitationResponse,
            413: CreateProductReviewAndSellerFeedbackSolicitationResponse,
            415: CreateProductReviewAndSellerFeedbackSolicitationResponse,
            429: CreateProductReviewAndSellerFeedbackSolicitationResponse,
            500: CreateProductReviewAndSellerFeedbackSolicitationResponse,
            503: CreateProductReviewAndSellerFeedbackSolicitationResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))
